/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSAccessibility;

import dstep.appkit.AppKitDefines;
import dstep.foundation.NSArray;
import dstep.appkit.NSErrors;
import dstep.foundation.NSString;
import dstep.foundation.NSGeometry;
import dstep.foundation.NSObject;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

import bindings = dstep.appkit.NSAccessibility_bindings;

private
{
	NSString NSAccessibilityErrorCodeExceptionInfo_;
	NSString NSAccessibilityRoleAttribute_;
	NSString NSAccessibilityRoleDescriptionAttribute_;
	NSString NSAccessibilitySubroleAttribute_;
	NSString NSAccessibilityHelpAttribute_;
	NSString NSAccessibilityValueAttribute_;
	NSString NSAccessibilityMinValueAttribute_;
	NSString NSAccessibilityMaxValueAttribute_;
	NSString NSAccessibilityEnabledAttribute_;
	NSString NSAccessibilityFocusedAttribute_;
	NSString NSAccessibilityParentAttribute_;
	NSString NSAccessibilityChildrenAttribute_;
	NSString NSAccessibilityWindowAttribute_;
	NSString NSAccessibilityTopLevelUIElementAttribute_;
	NSString NSAccessibilitySelectedChildrenAttribute_;
	NSString NSAccessibilityVisibleChildrenAttribute_;
	NSString NSAccessibilityPositionAttribute_;
	NSString NSAccessibilitySizeAttribute_;
	NSString NSAccessibilityContentsAttribute_;
	NSString NSAccessibilityTitleAttribute_;
	NSString NSAccessibilityDescriptionAttribute_;
	NSString NSAccessibilityShownMenuAttribute_;
	NSString NSAccessibilityValueDescriptionAttribute_;
	NSString NSAccessibilityPreviousContentsAttribute_;
	NSString NSAccessibilityNextContentsAttribute_;
	NSString NSAccessibilityHeaderAttribute_;
	NSString NSAccessibilityEditedAttribute_;
	NSString NSAccessibilityTabsAttribute_;
	NSString NSAccessibilityHorizontalScrollBarAttribute_;
	NSString NSAccessibilityVerticalScrollBarAttribute_;
	NSString NSAccessibilityOverflowButtonAttribute_;
	NSString NSAccessibilityIncrementButtonAttribute_;
	NSString NSAccessibilityDecrementButtonAttribute_;
	NSString NSAccessibilityFilenameAttribute_;
	NSString NSAccessibilityExpandedAttribute_;
	NSString NSAccessibilitySelectedAttribute_;
	NSString NSAccessibilitySplittersAttribute_;
	NSString NSAccessibilityDocumentAttribute_;
	NSString NSAccessibilityURLAttribute_;
	NSString NSAccessibilityIndexAttribute_;
	NSString NSAccessibilityRowCountAttribute_;
	NSString NSAccessibilityColumnCountAttribute_;
	NSString NSAccessibilityOrderedByRowAttribute_;
	NSString NSAccessibilityTitleUIElementAttribute_;
	NSString NSAccessibilityServesAsTitleForUIElementsAttribute_;
	NSString NSAccessibilityLinkedUIElementsAttribute_;
	NSString NSAccessibilitySelectedTextAttribute_;
	NSString NSAccessibilitySelectedTextRangeAttribute_;
	NSString NSAccessibilityNumberOfCharactersAttribute_;
	NSString NSAccessibilityVisibleCharacterRangeAttribute_;
	NSString NSAccessibilitySharedTextUIElementsAttribute_;
	NSString NSAccessibilitySharedCharacterRangeAttribute_;
	NSString NSAccessibilityInsertionPointLineNumberAttribute_;
	NSString NSAccessibilitySelectedTextRangesAttribute_;
	NSString NSAccessibilityLineForIndexParameterizedAttribute_;
	NSString NSAccessibilityRangeForLineParameterizedAttribute_;
	NSString NSAccessibilityStringForRangeParameterizedAttribute_;
	NSString NSAccessibilityRangeForPositionParameterizedAttribute_;
	NSString NSAccessibilityRangeForIndexParameterizedAttribute_;
	NSString NSAccessibilityBoundsForRangeParameterizedAttribute_;
	NSString NSAccessibilityRTFForRangeParameterizedAttribute_;
	NSString NSAccessibilityStyleRangeForIndexParameterizedAttribute_;
	NSString NSAccessibilityAttributedStringForRangeParameterizedAttribute_;
	NSString NSAccessibilityFontTextAttribute_;
	NSString NSAccessibilityForegroundColorTextAttribute_;
	NSString NSAccessibilityBackgroundColorTextAttribute_;
	NSString NSAccessibilityUnderlineColorTextAttribute_;
	NSString NSAccessibilityStrikethroughColorTextAttribute_;
	NSString NSAccessibilityUnderlineTextAttribute_;
	NSString NSAccessibilitySuperscriptTextAttribute_;
	NSString NSAccessibilityStrikethroughTextAttribute_;
	NSString NSAccessibilityShadowTextAttribute_;
	NSString NSAccessibilityAttachmentTextAttribute_;
	NSString NSAccessibilityLinkTextAttribute_;
	NSString NSAccessibilityMisspelledTextAttribute_;
	NSString NSAccessibilityFontNameKey_;
	NSString NSAccessibilityFontFamilyKey_;
	NSString NSAccessibilityVisibleNameKey_;
	NSString NSAccessibilityFontSizeKey_;
	NSString NSAccessibilityMainAttribute_;
	NSString NSAccessibilityMinimizedAttribute_;
	NSString NSAccessibilityCloseButtonAttribute_;
	NSString NSAccessibilityZoomButtonAttribute_;
	NSString NSAccessibilityMinimizeButtonAttribute_;
	NSString NSAccessibilityToolbarButtonAttribute_;
	NSString NSAccessibilityProxyAttribute_;
	NSString NSAccessibilityGrowAreaAttribute_;
	NSString NSAccessibilityModalAttribute_;
	NSString NSAccessibilityDefaultButtonAttribute_;
	NSString NSAccessibilityCancelButtonAttribute_;
	NSString NSAccessibilityMenuBarAttribute_;
	NSString NSAccessibilityWindowsAttribute_;
	NSString NSAccessibilityFrontmostAttribute_;
	NSString NSAccessibilityHiddenAttribute_;
	NSString NSAccessibilityMainWindowAttribute_;
	NSString NSAccessibilityFocusedWindowAttribute_;
	NSString NSAccessibilityFocusedUIElementAttribute_;
	NSString NSAccessibilityOrientationAttribute_;
	NSString NSAccessibilityVerticalOrientationValue_;
	NSString NSAccessibilityHorizontalOrientationValue_;
	NSString NSAccessibilityColumnTitlesAttribute_;
	NSString NSAccessibilitySearchButtonAttribute_;
	NSString NSAccessibilitySearchMenuAttribute_;
	NSString NSAccessibilityClearButtonAttribute_;
	NSString NSAccessibilityRowsAttribute_;
	NSString NSAccessibilityVisibleRowsAttribute_;
	NSString NSAccessibilitySelectedRowsAttribute_;
	NSString NSAccessibilityColumnsAttribute_;
	NSString NSAccessibilityVisibleColumnsAttribute_;
	NSString NSAccessibilitySelectedColumnsAttribute_;
	NSString NSAccessibilitySortDirectionAttribute_;
	NSString NSAccessibilityAscendingSortDirectionValue_;
	NSString NSAccessibilityDescendingSortDirectionValue_;
	NSString NSAccessibilityUnknownSortDirectionValue_;
	NSString NSAccessibilityDisclosingAttribute_;
	NSString NSAccessibilityDisclosedRowsAttribute_;
	NSString NSAccessibilityDisclosedByRowAttribute_;
	NSString NSAccessibilityDisclosureLevelAttribute_;
	NSString NSAccessibilityAllowedValuesAttribute_;
	NSString NSAccessibilityLabelUIElementsAttribute_;
	NSString NSAccessibilityLabelValueAttribute_;
	NSString NSAccessibilityMatteHoleAttribute_;
	NSString NSAccessibilityMatteContentUIElementAttribute_;
	NSString NSAccessibilityMarkerUIElementsAttribute_;
	NSString NSAccessibilityMarkerValuesAttribute_;
	NSString NSAccessibilityMarkerGroupUIElementAttribute_;
	NSString NSAccessibilityUnitsAttribute_;
	NSString NSAccessibilityUnitDescriptionAttribute_;
	NSString NSAccessibilityMarkerTypeAttribute_;
	NSString NSAccessibilityMarkerTypeDescriptionAttribute_;
	NSString NSAccessibilityLeftTabStopMarkerTypeValue_;
	NSString NSAccessibilityRightTabStopMarkerTypeValue_;
	NSString NSAccessibilityCenterTabStopMarkerTypeValue_;
	NSString NSAccessibilityDecimalTabStopMarkerTypeValue_;
	NSString NSAccessibilityHeadIndentMarkerTypeValue_;
	NSString NSAccessibilityTailIndentMarkerTypeValue_;
	NSString NSAccessibilityFirstLineIndentMarkerTypeValue_;
	NSString NSAccessibilityUnknownMarkerTypeValue_;
	NSString NSAccessibilityInchesUnitValue_;
	NSString NSAccessibilityCentimetersUnitValue_;
	NSString NSAccessibilityPointsUnitValue_;
	NSString NSAccessibilityPicasUnitValue_;
	NSString NSAccessibilityUnknownUnitValue_;
	NSString NSAccessibilityPressAction_;
	NSString NSAccessibilityIncrementAction_;
	NSString NSAccessibilityDecrementAction_;
	NSString NSAccessibilityConfirmAction_;
	NSString NSAccessibilityPickAction_;
	NSString NSAccessibilityCancelAction_;
	NSString NSAccessibilityRaiseAction_;
	NSString NSAccessibilityShowMenuAction_;
	NSString NSAccessibilityDeleteAction_;
	NSString NSAccessibilityMainWindowChangedNotification_;
	NSString NSAccessibilityFocusedWindowChangedNotification_;
	NSString NSAccessibilityFocusedUIElementChangedNotification_;
	NSString NSAccessibilityApplicationActivatedNotification_;
	NSString NSAccessibilityApplicationDeactivatedNotification_;
	NSString NSAccessibilityApplicationHiddenNotification_;
	NSString NSAccessibilityApplicationShownNotification_;
	NSString NSAccessibilityWindowCreatedNotification_;
	NSString NSAccessibilityWindowMovedNotification_;
	NSString NSAccessibilityWindowResizedNotification_;
	NSString NSAccessibilityWindowMiniaturizedNotification_;
	NSString NSAccessibilityWindowDeminiaturizedNotification_;
	NSString NSAccessibilityDrawerCreatedNotification_;
	NSString NSAccessibilitySheetCreatedNotification_;
	NSString NSAccessibilityUIElementDestroyedNotification_;
	NSString NSAccessibilityValueChangedNotification_;
	NSString NSAccessibilityTitleChangedNotification_;
	NSString NSAccessibilityResizedNotification_;
	NSString NSAccessibilityMovedNotification_;
	NSString NSAccessibilityCreatedNotification_;
	NSString NSAccessibilityHelpTagCreatedNotification_;
	NSString NSAccessibilitySelectedTextChangedNotification_;
	NSString NSAccessibilityRowCountChangedNotification_;
	NSString NSAccessibilitySelectedChildrenChangedNotification_;
	NSString NSAccessibilitySelectedRowsChangedNotification_;
	NSString NSAccessibilitySelectedColumnsChangedNotification_;
	NSString NSAccessibilityUnknownRole_;
	NSString NSAccessibilityButtonRole_;
	NSString NSAccessibilityRadioButtonRole_;
	NSString NSAccessibilityCheckBoxRole_;
	NSString NSAccessibilitySliderRole_;
	NSString NSAccessibilityTabGroupRole_;
	NSString NSAccessibilityTextFieldRole_;
	NSString NSAccessibilityStaticTextRole_;
	NSString NSAccessibilityTextAreaRole_;
	NSString NSAccessibilityScrollAreaRole_;
	NSString NSAccessibilityPopUpButtonRole_;
	NSString NSAccessibilityMenuButtonRole_;
	NSString NSAccessibilityTableRole_;
	NSString NSAccessibilityApplicationRole_;
	NSString NSAccessibilityGroupRole_;
	NSString NSAccessibilityRadioGroupRole_;
	NSString NSAccessibilityListRole_;
	NSString NSAccessibilityScrollBarRole_;
	NSString NSAccessibilityValueIndicatorRole_;
	NSString NSAccessibilityImageRole_;
	NSString NSAccessibilityMenuBarRole_;
	NSString NSAccessibilityMenuRole_;
	NSString NSAccessibilityMenuItemRole_;
	NSString NSAccessibilityColumnRole_;
	NSString NSAccessibilityRowRole_;
	NSString NSAccessibilityToolbarRole_;
	NSString NSAccessibilityBusyIndicatorRole_;
	NSString NSAccessibilityProgressIndicatorRole_;
	NSString NSAccessibilityWindowRole_;
	NSString NSAccessibilityDrawerRole_;
	NSString NSAccessibilitySystemWideRole_;
	NSString NSAccessibilityOutlineRole_;
	NSString NSAccessibilityIncrementorRole_;
	NSString NSAccessibilityBrowserRole_;
	NSString NSAccessibilityComboBoxRole_;
	NSString NSAccessibilitySplitGroupRole_;
	NSString NSAccessibilitySplitterRole_;
	NSString NSAccessibilityColorWellRole_;
	NSString NSAccessibilityGrowAreaRole_;
	NSString NSAccessibilitySheetRole_;
	NSString NSAccessibilityHelpTagRole_;
	NSString NSAccessibilityMatteRole_;
	NSString NSAccessibilityRulerRole_;
	NSString NSAccessibilityRulerMarkerRole_;
	NSString NSAccessibilitySortButtonRole_;
	NSString NSAccessibilityLinkRole_;
	NSString NSAccessibilityDisclosureTriangleRole_;
	NSString NSAccessibilityGridRole_;
	NSString NSAccessibilityUnknownSubrole_;
	NSString NSAccessibilityCloseButtonSubrole_;
	NSString NSAccessibilityZoomButtonSubrole_;
	NSString NSAccessibilityMinimizeButtonSubrole_;
	NSString NSAccessibilityToolbarButtonSubrole_;
	NSString NSAccessibilityTableRowSubrole_;
	NSString NSAccessibilityOutlineRowSubrole_;
	NSString NSAccessibilitySecureTextFieldSubrole_;
	NSString NSAccessibilityStandardWindowSubrole_;
	NSString NSAccessibilityDialogSubrole_;
	NSString NSAccessibilitySystemDialogSubrole_;
	NSString NSAccessibilityFloatingWindowSubrole_;
	NSString NSAccessibilitySystemFloatingWindowSubrole_;
	NSString NSAccessibilityIncrementArrowSubrole_;
	NSString NSAccessibilityDecrementArrowSubrole_;
	NSString NSAccessibilityIncrementPageSubrole_;
	NSString NSAccessibilityDecrementPageSubrole_;
	NSString NSAccessibilitySearchFieldSubrole_;
	NSString NSAccessibilityTextAttachmentSubrole_;
	NSString NSAccessibilityTextLinkSubrole_;
	NSString NSAccessibilityTimelineSubrole_;
}

NSString NSAccessibilityErrorCodeExceptionInfo ()
{
	if (NSAccessibilityErrorCodeExceptionInfo_)
		return NSAccessibilityErrorCodeExceptionInfo_;

	return NSAccessibilityErrorCodeExceptionInfo_ = new NSString(bindings.NSAccessibilityErrorCodeExceptionInfo);
}

NSString NSAccessibilityRoleAttribute ()
{
	if (NSAccessibilityRoleAttribute_)
		return NSAccessibilityRoleAttribute_;

	return NSAccessibilityRoleAttribute_ = new NSString(bindings.NSAccessibilityRoleAttribute);
}

NSString NSAccessibilityRoleDescriptionAttribute ()
{
	if (NSAccessibilityRoleDescriptionAttribute_)
		return NSAccessibilityRoleDescriptionAttribute_;

	return NSAccessibilityRoleDescriptionAttribute_ = new NSString(bindings.NSAccessibilityRoleDescriptionAttribute);
}

NSString NSAccessibilitySubroleAttribute ()
{
	if (NSAccessibilitySubroleAttribute_)
		return NSAccessibilitySubroleAttribute_;

	return NSAccessibilitySubroleAttribute_ = new NSString(bindings.NSAccessibilitySubroleAttribute);
}

NSString NSAccessibilityHelpAttribute ()
{
	if (NSAccessibilityHelpAttribute_)
		return NSAccessibilityHelpAttribute_;

	return NSAccessibilityHelpAttribute_ = new NSString(bindings.NSAccessibilityHelpAttribute);
}

NSString NSAccessibilityValueAttribute ()
{
	if (NSAccessibilityValueAttribute_)
		return NSAccessibilityValueAttribute_;

	return NSAccessibilityValueAttribute_ = new NSString(bindings.NSAccessibilityValueAttribute);
}

NSString NSAccessibilityMinValueAttribute ()
{
	if (NSAccessibilityMinValueAttribute_)
		return NSAccessibilityMinValueAttribute_;

	return NSAccessibilityMinValueAttribute_ = new NSString(bindings.NSAccessibilityMinValueAttribute);
}

NSString NSAccessibilityMaxValueAttribute ()
{
	if (NSAccessibilityMaxValueAttribute_)
		return NSAccessibilityMaxValueAttribute_;

	return NSAccessibilityMaxValueAttribute_ = new NSString(bindings.NSAccessibilityMaxValueAttribute);
}

NSString NSAccessibilityEnabledAttribute ()
{
	if (NSAccessibilityEnabledAttribute_)
		return NSAccessibilityEnabledAttribute_;

	return NSAccessibilityEnabledAttribute_ = new NSString(bindings.NSAccessibilityEnabledAttribute);
}

NSString NSAccessibilityFocusedAttribute ()
{
	if (NSAccessibilityFocusedAttribute_)
		return NSAccessibilityFocusedAttribute_;

	return NSAccessibilityFocusedAttribute_ = new NSString(bindings.NSAccessibilityFocusedAttribute);
}

NSString NSAccessibilityParentAttribute ()
{
	if (NSAccessibilityParentAttribute_)
		return NSAccessibilityParentAttribute_;

	return NSAccessibilityParentAttribute_ = new NSString(bindings.NSAccessibilityParentAttribute);
}

NSString NSAccessibilityChildrenAttribute ()
{
	if (NSAccessibilityChildrenAttribute_)
		return NSAccessibilityChildrenAttribute_;

	return NSAccessibilityChildrenAttribute_ = new NSString(bindings.NSAccessibilityChildrenAttribute);
}

NSString NSAccessibilityWindowAttribute ()
{
	if (NSAccessibilityWindowAttribute_)
		return NSAccessibilityWindowAttribute_;

	return NSAccessibilityWindowAttribute_ = new NSString(bindings.NSAccessibilityWindowAttribute);
}

NSString NSAccessibilityTopLevelUIElementAttribute ()
{
	if (NSAccessibilityTopLevelUIElementAttribute_)
		return NSAccessibilityTopLevelUIElementAttribute_;

	return NSAccessibilityTopLevelUIElementAttribute_ = new NSString(bindings.NSAccessibilityTopLevelUIElementAttribute);
}

NSString NSAccessibilitySelectedChildrenAttribute ()
{
	if (NSAccessibilitySelectedChildrenAttribute_)
		return NSAccessibilitySelectedChildrenAttribute_;

	return NSAccessibilitySelectedChildrenAttribute_ = new NSString(bindings.NSAccessibilitySelectedChildrenAttribute);
}

NSString NSAccessibilityVisibleChildrenAttribute ()
{
	if (NSAccessibilityVisibleChildrenAttribute_)
		return NSAccessibilityVisibleChildrenAttribute_;

	return NSAccessibilityVisibleChildrenAttribute_ = new NSString(bindings.NSAccessibilityVisibleChildrenAttribute);
}

NSString NSAccessibilityPositionAttribute ()
{
	if (NSAccessibilityPositionAttribute_)
		return NSAccessibilityPositionAttribute_;

	return NSAccessibilityPositionAttribute_ = new NSString(bindings.NSAccessibilityPositionAttribute);
}

NSString NSAccessibilitySizeAttribute ()
{
	if (NSAccessibilitySizeAttribute_)
		return NSAccessibilitySizeAttribute_;

	return NSAccessibilitySizeAttribute_ = new NSString(bindings.NSAccessibilitySizeAttribute);
}

NSString NSAccessibilityContentsAttribute ()
{
	if (NSAccessibilityContentsAttribute_)
		return NSAccessibilityContentsAttribute_;

	return NSAccessibilityContentsAttribute_ = new NSString(bindings.NSAccessibilityContentsAttribute);
}

NSString NSAccessibilityTitleAttribute ()
{
	if (NSAccessibilityTitleAttribute_)
		return NSAccessibilityTitleAttribute_;

	return NSAccessibilityTitleAttribute_ = new NSString(bindings.NSAccessibilityTitleAttribute);
}

NSString NSAccessibilityDescriptionAttribute ()
{
	if (NSAccessibilityDescriptionAttribute_)
		return NSAccessibilityDescriptionAttribute_;

	return NSAccessibilityDescriptionAttribute_ = new NSString(bindings.NSAccessibilityDescriptionAttribute);
}

NSString NSAccessibilityShownMenuAttribute ()
{
	if (NSAccessibilityShownMenuAttribute_)
		return NSAccessibilityShownMenuAttribute_;

	return NSAccessibilityShownMenuAttribute_ = new NSString(bindings.NSAccessibilityShownMenuAttribute);
}

NSString NSAccessibilityValueDescriptionAttribute ()
{
	if (NSAccessibilityValueDescriptionAttribute_)
		return NSAccessibilityValueDescriptionAttribute_;

	return NSAccessibilityValueDescriptionAttribute_ = new NSString(bindings.NSAccessibilityValueDescriptionAttribute);
}

NSString NSAccessibilityPreviousContentsAttribute ()
{
	if (NSAccessibilityPreviousContentsAttribute_)
		return NSAccessibilityPreviousContentsAttribute_;

	return NSAccessibilityPreviousContentsAttribute_ = new NSString(bindings.NSAccessibilityPreviousContentsAttribute);
}

NSString NSAccessibilityNextContentsAttribute ()
{
	if (NSAccessibilityNextContentsAttribute_)
		return NSAccessibilityNextContentsAttribute_;

	return NSAccessibilityNextContentsAttribute_ = new NSString(bindings.NSAccessibilityNextContentsAttribute);
}

NSString NSAccessibilityHeaderAttribute ()
{
	if (NSAccessibilityHeaderAttribute_)
		return NSAccessibilityHeaderAttribute_;

	return NSAccessibilityHeaderAttribute_ = new NSString(bindings.NSAccessibilityHeaderAttribute);
}

NSString NSAccessibilityEditedAttribute ()
{
	if (NSAccessibilityEditedAttribute_)
		return NSAccessibilityEditedAttribute_;

	return NSAccessibilityEditedAttribute_ = new NSString(bindings.NSAccessibilityEditedAttribute);
}

NSString NSAccessibilityTabsAttribute ()
{
	if (NSAccessibilityTabsAttribute_)
		return NSAccessibilityTabsAttribute_;

	return NSAccessibilityTabsAttribute_ = new NSString(bindings.NSAccessibilityTabsAttribute);
}

NSString NSAccessibilityHorizontalScrollBarAttribute ()
{
	if (NSAccessibilityHorizontalScrollBarAttribute_)
		return NSAccessibilityHorizontalScrollBarAttribute_;

	return NSAccessibilityHorizontalScrollBarAttribute_ = new NSString(bindings.NSAccessibilityHorizontalScrollBarAttribute);
}

NSString NSAccessibilityVerticalScrollBarAttribute ()
{
	if (NSAccessibilityVerticalScrollBarAttribute_)
		return NSAccessibilityVerticalScrollBarAttribute_;

	return NSAccessibilityVerticalScrollBarAttribute_ = new NSString(bindings.NSAccessibilityVerticalScrollBarAttribute);
}

NSString NSAccessibilityOverflowButtonAttribute ()
{
	if (NSAccessibilityOverflowButtonAttribute_)
		return NSAccessibilityOverflowButtonAttribute_;

	return NSAccessibilityOverflowButtonAttribute_ = new NSString(bindings.NSAccessibilityOverflowButtonAttribute);
}

NSString NSAccessibilityIncrementButtonAttribute ()
{
	if (NSAccessibilityIncrementButtonAttribute_)
		return NSAccessibilityIncrementButtonAttribute_;

	return NSAccessibilityIncrementButtonAttribute_ = new NSString(bindings.NSAccessibilityIncrementButtonAttribute);
}

NSString NSAccessibilityDecrementButtonAttribute ()
{
	if (NSAccessibilityDecrementButtonAttribute_)
		return NSAccessibilityDecrementButtonAttribute_;

	return NSAccessibilityDecrementButtonAttribute_ = new NSString(bindings.NSAccessibilityDecrementButtonAttribute);
}

NSString NSAccessibilityFilenameAttribute ()
{
	if (NSAccessibilityFilenameAttribute_)
		return NSAccessibilityFilenameAttribute_;

	return NSAccessibilityFilenameAttribute_ = new NSString(bindings.NSAccessibilityFilenameAttribute);
}

NSString NSAccessibilityExpandedAttribute ()
{
	if (NSAccessibilityExpandedAttribute_)
		return NSAccessibilityExpandedAttribute_;

	return NSAccessibilityExpandedAttribute_ = new NSString(bindings.NSAccessibilityExpandedAttribute);
}

NSString NSAccessibilitySelectedAttribute ()
{
	if (NSAccessibilitySelectedAttribute_)
		return NSAccessibilitySelectedAttribute_;

	return NSAccessibilitySelectedAttribute_ = new NSString(bindings.NSAccessibilitySelectedAttribute);
}

NSString NSAccessibilitySplittersAttribute ()
{
	if (NSAccessibilitySplittersAttribute_)
		return NSAccessibilitySplittersAttribute_;

	return NSAccessibilitySplittersAttribute_ = new NSString(bindings.NSAccessibilitySplittersAttribute);
}

NSString NSAccessibilityDocumentAttribute ()
{
	if (NSAccessibilityDocumentAttribute_)
		return NSAccessibilityDocumentAttribute_;

	return NSAccessibilityDocumentAttribute_ = new NSString(bindings.NSAccessibilityDocumentAttribute);
}

NSString NSAccessibilityURLAttribute ()
{
	if (NSAccessibilityURLAttribute_)
		return NSAccessibilityURLAttribute_;

	return NSAccessibilityURLAttribute_ = new NSString(bindings.NSAccessibilityURLAttribute);
}

NSString NSAccessibilityIndexAttribute ()
{
	if (NSAccessibilityIndexAttribute_)
		return NSAccessibilityIndexAttribute_;

	return NSAccessibilityIndexAttribute_ = new NSString(bindings.NSAccessibilityIndexAttribute);
}

NSString NSAccessibilityRowCountAttribute ()
{
	if (NSAccessibilityRowCountAttribute_)
		return NSAccessibilityRowCountAttribute_;

	return NSAccessibilityRowCountAttribute_ = new NSString(bindings.NSAccessibilityRowCountAttribute);
}

NSString NSAccessibilityColumnCountAttribute ()
{
	if (NSAccessibilityColumnCountAttribute_)
		return NSAccessibilityColumnCountAttribute_;

	return NSAccessibilityColumnCountAttribute_ = new NSString(bindings.NSAccessibilityColumnCountAttribute);
}

NSString NSAccessibilityOrderedByRowAttribute ()
{
	if (NSAccessibilityOrderedByRowAttribute_)
		return NSAccessibilityOrderedByRowAttribute_;

	return NSAccessibilityOrderedByRowAttribute_ = new NSString(bindings.NSAccessibilityOrderedByRowAttribute);
}

NSString NSAccessibilityTitleUIElementAttribute ()
{
	if (NSAccessibilityTitleUIElementAttribute_)
		return NSAccessibilityTitleUIElementAttribute_;

	return NSAccessibilityTitleUIElementAttribute_ = new NSString(bindings.NSAccessibilityTitleUIElementAttribute);
}

NSString NSAccessibilityServesAsTitleForUIElementsAttribute ()
{
	if (NSAccessibilityServesAsTitleForUIElementsAttribute_)
		return NSAccessibilityServesAsTitleForUIElementsAttribute_;

	return NSAccessibilityServesAsTitleForUIElementsAttribute_ = new NSString(bindings.NSAccessibilityServesAsTitleForUIElementsAttribute);
}

NSString NSAccessibilityLinkedUIElementsAttribute ()
{
	if (NSAccessibilityLinkedUIElementsAttribute_)
		return NSAccessibilityLinkedUIElementsAttribute_;

	return NSAccessibilityLinkedUIElementsAttribute_ = new NSString(bindings.NSAccessibilityLinkedUIElementsAttribute);
}

NSString NSAccessibilitySelectedTextAttribute ()
{
	if (NSAccessibilitySelectedTextAttribute_)
		return NSAccessibilitySelectedTextAttribute_;

	return NSAccessibilitySelectedTextAttribute_ = new NSString(bindings.NSAccessibilitySelectedTextAttribute);
}

NSString NSAccessibilitySelectedTextRangeAttribute ()
{
	if (NSAccessibilitySelectedTextRangeAttribute_)
		return NSAccessibilitySelectedTextRangeAttribute_;

	return NSAccessibilitySelectedTextRangeAttribute_ = new NSString(bindings.NSAccessibilitySelectedTextRangeAttribute);
}

NSString NSAccessibilityNumberOfCharactersAttribute ()
{
	if (NSAccessibilityNumberOfCharactersAttribute_)
		return NSAccessibilityNumberOfCharactersAttribute_;

	return NSAccessibilityNumberOfCharactersAttribute_ = new NSString(bindings.NSAccessibilityNumberOfCharactersAttribute);
}

NSString NSAccessibilityVisibleCharacterRangeAttribute ()
{
	if (NSAccessibilityVisibleCharacterRangeAttribute_)
		return NSAccessibilityVisibleCharacterRangeAttribute_;

	return NSAccessibilityVisibleCharacterRangeAttribute_ = new NSString(bindings.NSAccessibilityVisibleCharacterRangeAttribute);
}

NSString NSAccessibilitySharedTextUIElementsAttribute ()
{
	if (NSAccessibilitySharedTextUIElementsAttribute_)
		return NSAccessibilitySharedTextUIElementsAttribute_;

	return NSAccessibilitySharedTextUIElementsAttribute_ = new NSString(bindings.NSAccessibilitySharedTextUIElementsAttribute);
}

NSString NSAccessibilitySharedCharacterRangeAttribute ()
{
	if (NSAccessibilitySharedCharacterRangeAttribute_)
		return NSAccessibilitySharedCharacterRangeAttribute_;

	return NSAccessibilitySharedCharacterRangeAttribute_ = new NSString(bindings.NSAccessibilitySharedCharacterRangeAttribute);
}

NSString NSAccessibilityInsertionPointLineNumberAttribute ()
{
	if (NSAccessibilityInsertionPointLineNumberAttribute_)
		return NSAccessibilityInsertionPointLineNumberAttribute_;

	return NSAccessibilityInsertionPointLineNumberAttribute_ = new NSString(bindings.NSAccessibilityInsertionPointLineNumberAttribute);
}

NSString NSAccessibilitySelectedTextRangesAttribute ()
{
	if (NSAccessibilitySelectedTextRangesAttribute_)
		return NSAccessibilitySelectedTextRangesAttribute_;

	return NSAccessibilitySelectedTextRangesAttribute_ = new NSString(bindings.NSAccessibilitySelectedTextRangesAttribute);
}

NSString NSAccessibilityLineForIndexParameterizedAttribute ()
{
	if (NSAccessibilityLineForIndexParameterizedAttribute_)
		return NSAccessibilityLineForIndexParameterizedAttribute_;

	return NSAccessibilityLineForIndexParameterizedAttribute_ = new NSString(bindings.NSAccessibilityLineForIndexParameterizedAttribute);
}

NSString NSAccessibilityRangeForLineParameterizedAttribute ()
{
	if (NSAccessibilityRangeForLineParameterizedAttribute_)
		return NSAccessibilityRangeForLineParameterizedAttribute_;

	return NSAccessibilityRangeForLineParameterizedAttribute_ = new NSString(bindings.NSAccessibilityRangeForLineParameterizedAttribute);
}

NSString NSAccessibilityStringForRangeParameterizedAttribute ()
{
	if (NSAccessibilityStringForRangeParameterizedAttribute_)
		return NSAccessibilityStringForRangeParameterizedAttribute_;

	return NSAccessibilityStringForRangeParameterizedAttribute_ = new NSString(bindings.NSAccessibilityStringForRangeParameterizedAttribute);
}

NSString NSAccessibilityRangeForPositionParameterizedAttribute ()
{
	if (NSAccessibilityRangeForPositionParameterizedAttribute_)
		return NSAccessibilityRangeForPositionParameterizedAttribute_;

	return NSAccessibilityRangeForPositionParameterizedAttribute_ = new NSString(bindings.NSAccessibilityRangeForPositionParameterizedAttribute);
}

NSString NSAccessibilityRangeForIndexParameterizedAttribute ()
{
	if (NSAccessibilityRangeForIndexParameterizedAttribute_)
		return NSAccessibilityRangeForIndexParameterizedAttribute_;

	return NSAccessibilityRangeForIndexParameterizedAttribute_ = new NSString(bindings.NSAccessibilityRangeForIndexParameterizedAttribute);
}

NSString NSAccessibilityBoundsForRangeParameterizedAttribute ()
{
	if (NSAccessibilityBoundsForRangeParameterizedAttribute_)
		return NSAccessibilityBoundsForRangeParameterizedAttribute_;

	return NSAccessibilityBoundsForRangeParameterizedAttribute_ = new NSString(bindings.NSAccessibilityBoundsForRangeParameterizedAttribute);
}

NSString NSAccessibilityRTFForRangeParameterizedAttribute ()
{
	if (NSAccessibilityRTFForRangeParameterizedAttribute_)
		return NSAccessibilityRTFForRangeParameterizedAttribute_;

	return NSAccessibilityRTFForRangeParameterizedAttribute_ = new NSString(bindings.NSAccessibilityRTFForRangeParameterizedAttribute);
}

NSString NSAccessibilityStyleRangeForIndexParameterizedAttribute ()
{
	if (NSAccessibilityStyleRangeForIndexParameterizedAttribute_)
		return NSAccessibilityStyleRangeForIndexParameterizedAttribute_;

	return NSAccessibilityStyleRangeForIndexParameterizedAttribute_ = new NSString(bindings.NSAccessibilityStyleRangeForIndexParameterizedAttribute);
}

NSString NSAccessibilityAttributedStringForRangeParameterizedAttribute ()
{
	if (NSAccessibilityAttributedStringForRangeParameterizedAttribute_)
		return NSAccessibilityAttributedStringForRangeParameterizedAttribute_;

	return NSAccessibilityAttributedStringForRangeParameterizedAttribute_ = new NSString(bindings.NSAccessibilityAttributedStringForRangeParameterizedAttribute);
}

NSString NSAccessibilityFontTextAttribute ()
{
	if (NSAccessibilityFontTextAttribute_)
		return NSAccessibilityFontTextAttribute_;

	return NSAccessibilityFontTextAttribute_ = new NSString(bindings.NSAccessibilityFontTextAttribute);
}

NSString NSAccessibilityForegroundColorTextAttribute ()
{
	if (NSAccessibilityForegroundColorTextAttribute_)
		return NSAccessibilityForegroundColorTextAttribute_;

	return NSAccessibilityForegroundColorTextAttribute_ = new NSString(bindings.NSAccessibilityForegroundColorTextAttribute);
}

NSString NSAccessibilityBackgroundColorTextAttribute ()
{
	if (NSAccessibilityBackgroundColorTextAttribute_)
		return NSAccessibilityBackgroundColorTextAttribute_;

	return NSAccessibilityBackgroundColorTextAttribute_ = new NSString(bindings.NSAccessibilityBackgroundColorTextAttribute);
}

NSString NSAccessibilityUnderlineColorTextAttribute ()
{
	if (NSAccessibilityUnderlineColorTextAttribute_)
		return NSAccessibilityUnderlineColorTextAttribute_;

	return NSAccessibilityUnderlineColorTextAttribute_ = new NSString(bindings.NSAccessibilityUnderlineColorTextAttribute);
}

NSString NSAccessibilityStrikethroughColorTextAttribute ()
{
	if (NSAccessibilityStrikethroughColorTextAttribute_)
		return NSAccessibilityStrikethroughColorTextAttribute_;

	return NSAccessibilityStrikethroughColorTextAttribute_ = new NSString(bindings.NSAccessibilityStrikethroughColorTextAttribute);
}

NSString NSAccessibilityUnderlineTextAttribute ()
{
	if (NSAccessibilityUnderlineTextAttribute_)
		return NSAccessibilityUnderlineTextAttribute_;

	return NSAccessibilityUnderlineTextAttribute_ = new NSString(bindings.NSAccessibilityUnderlineTextAttribute);
}

NSString NSAccessibilitySuperscriptTextAttribute ()
{
	if (NSAccessibilitySuperscriptTextAttribute_)
		return NSAccessibilitySuperscriptTextAttribute_;

	return NSAccessibilitySuperscriptTextAttribute_ = new NSString(bindings.NSAccessibilitySuperscriptTextAttribute);
}

NSString NSAccessibilityStrikethroughTextAttribute ()
{
	if (NSAccessibilityStrikethroughTextAttribute_)
		return NSAccessibilityStrikethroughTextAttribute_;

	return NSAccessibilityStrikethroughTextAttribute_ = new NSString(bindings.NSAccessibilityStrikethroughTextAttribute);
}

NSString NSAccessibilityShadowTextAttribute ()
{
	if (NSAccessibilityShadowTextAttribute_)
		return NSAccessibilityShadowTextAttribute_;

	return NSAccessibilityShadowTextAttribute_ = new NSString(bindings.NSAccessibilityShadowTextAttribute);
}

NSString NSAccessibilityAttachmentTextAttribute ()
{
	if (NSAccessibilityAttachmentTextAttribute_)
		return NSAccessibilityAttachmentTextAttribute_;

	return NSAccessibilityAttachmentTextAttribute_ = new NSString(bindings.NSAccessibilityAttachmentTextAttribute);
}

NSString NSAccessibilityLinkTextAttribute ()
{
	if (NSAccessibilityLinkTextAttribute_)
		return NSAccessibilityLinkTextAttribute_;

	return NSAccessibilityLinkTextAttribute_ = new NSString(bindings.NSAccessibilityLinkTextAttribute);
}

NSString NSAccessibilityMisspelledTextAttribute ()
{
	if (NSAccessibilityMisspelledTextAttribute_)
		return NSAccessibilityMisspelledTextAttribute_;

	return NSAccessibilityMisspelledTextAttribute_ = new NSString(bindings.NSAccessibilityMisspelledTextAttribute);
}

NSString NSAccessibilityFontNameKey ()
{
	if (NSAccessibilityFontNameKey_)
		return NSAccessibilityFontNameKey_;

	return NSAccessibilityFontNameKey_ = new NSString(bindings.NSAccessibilityFontNameKey);
}

NSString NSAccessibilityFontFamilyKey ()
{
	if (NSAccessibilityFontFamilyKey_)
		return NSAccessibilityFontFamilyKey_;

	return NSAccessibilityFontFamilyKey_ = new NSString(bindings.NSAccessibilityFontFamilyKey);
}

NSString NSAccessibilityVisibleNameKey ()
{
	if (NSAccessibilityVisibleNameKey_)
		return NSAccessibilityVisibleNameKey_;

	return NSAccessibilityVisibleNameKey_ = new NSString(bindings.NSAccessibilityVisibleNameKey);
}

NSString NSAccessibilityFontSizeKey ()
{
	if (NSAccessibilityFontSizeKey_)
		return NSAccessibilityFontSizeKey_;

	return NSAccessibilityFontSizeKey_ = new NSString(bindings.NSAccessibilityFontSizeKey);
}

NSString NSAccessibilityMainAttribute ()
{
	if (NSAccessibilityMainAttribute_)
		return NSAccessibilityMainAttribute_;

	return NSAccessibilityMainAttribute_ = new NSString(bindings.NSAccessibilityMainAttribute);
}

NSString NSAccessibilityMinimizedAttribute ()
{
	if (NSAccessibilityMinimizedAttribute_)
		return NSAccessibilityMinimizedAttribute_;

	return NSAccessibilityMinimizedAttribute_ = new NSString(bindings.NSAccessibilityMinimizedAttribute);
}

NSString NSAccessibilityCloseButtonAttribute ()
{
	if (NSAccessibilityCloseButtonAttribute_)
		return NSAccessibilityCloseButtonAttribute_;

	return NSAccessibilityCloseButtonAttribute_ = new NSString(bindings.NSAccessibilityCloseButtonAttribute);
}

NSString NSAccessibilityZoomButtonAttribute ()
{
	if (NSAccessibilityZoomButtonAttribute_)
		return NSAccessibilityZoomButtonAttribute_;

	return NSAccessibilityZoomButtonAttribute_ = new NSString(bindings.NSAccessibilityZoomButtonAttribute);
}

NSString NSAccessibilityMinimizeButtonAttribute ()
{
	if (NSAccessibilityMinimizeButtonAttribute_)
		return NSAccessibilityMinimizeButtonAttribute_;

	return NSAccessibilityMinimizeButtonAttribute_ = new NSString(bindings.NSAccessibilityMinimizeButtonAttribute);
}

NSString NSAccessibilityToolbarButtonAttribute ()
{
	if (NSAccessibilityToolbarButtonAttribute_)
		return NSAccessibilityToolbarButtonAttribute_;

	return NSAccessibilityToolbarButtonAttribute_ = new NSString(bindings.NSAccessibilityToolbarButtonAttribute);
}

NSString NSAccessibilityProxyAttribute ()
{
	if (NSAccessibilityProxyAttribute_)
		return NSAccessibilityProxyAttribute_;

	return NSAccessibilityProxyAttribute_ = new NSString(bindings.NSAccessibilityProxyAttribute);
}

NSString NSAccessibilityGrowAreaAttribute ()
{
	if (NSAccessibilityGrowAreaAttribute_)
		return NSAccessibilityGrowAreaAttribute_;

	return NSAccessibilityGrowAreaAttribute_ = new NSString(bindings.NSAccessibilityGrowAreaAttribute);
}

NSString NSAccessibilityModalAttribute ()
{
	if (NSAccessibilityModalAttribute_)
		return NSAccessibilityModalAttribute_;

	return NSAccessibilityModalAttribute_ = new NSString(bindings.NSAccessibilityModalAttribute);
}

NSString NSAccessibilityDefaultButtonAttribute ()
{
	if (NSAccessibilityDefaultButtonAttribute_)
		return NSAccessibilityDefaultButtonAttribute_;

	return NSAccessibilityDefaultButtonAttribute_ = new NSString(bindings.NSAccessibilityDefaultButtonAttribute);
}

NSString NSAccessibilityCancelButtonAttribute ()
{
	if (NSAccessibilityCancelButtonAttribute_)
		return NSAccessibilityCancelButtonAttribute_;

	return NSAccessibilityCancelButtonAttribute_ = new NSString(bindings.NSAccessibilityCancelButtonAttribute);
}

NSString NSAccessibilityMenuBarAttribute ()
{
	if (NSAccessibilityMenuBarAttribute_)
		return NSAccessibilityMenuBarAttribute_;

	return NSAccessibilityMenuBarAttribute_ = new NSString(bindings.NSAccessibilityMenuBarAttribute);
}

NSString NSAccessibilityWindowsAttribute ()
{
	if (NSAccessibilityWindowsAttribute_)
		return NSAccessibilityWindowsAttribute_;

	return NSAccessibilityWindowsAttribute_ = new NSString(bindings.NSAccessibilityWindowsAttribute);
}

NSString NSAccessibilityFrontmostAttribute ()
{
	if (NSAccessibilityFrontmostAttribute_)
		return NSAccessibilityFrontmostAttribute_;

	return NSAccessibilityFrontmostAttribute_ = new NSString(bindings.NSAccessibilityFrontmostAttribute);
}

NSString NSAccessibilityHiddenAttribute ()
{
	if (NSAccessibilityHiddenAttribute_)
		return NSAccessibilityHiddenAttribute_;

	return NSAccessibilityHiddenAttribute_ = new NSString(bindings.NSAccessibilityHiddenAttribute);
}

NSString NSAccessibilityMainWindowAttribute ()
{
	if (NSAccessibilityMainWindowAttribute_)
		return NSAccessibilityMainWindowAttribute_;

	return NSAccessibilityMainWindowAttribute_ = new NSString(bindings.NSAccessibilityMainWindowAttribute);
}

NSString NSAccessibilityFocusedWindowAttribute ()
{
	if (NSAccessibilityFocusedWindowAttribute_)
		return NSAccessibilityFocusedWindowAttribute_;

	return NSAccessibilityFocusedWindowAttribute_ = new NSString(bindings.NSAccessibilityFocusedWindowAttribute);
}

NSString NSAccessibilityFocusedUIElementAttribute ()
{
	if (NSAccessibilityFocusedUIElementAttribute_)
		return NSAccessibilityFocusedUIElementAttribute_;

	return NSAccessibilityFocusedUIElementAttribute_ = new NSString(bindings.NSAccessibilityFocusedUIElementAttribute);
}

NSString NSAccessibilityOrientationAttribute ()
{
	if (NSAccessibilityOrientationAttribute_)
		return NSAccessibilityOrientationAttribute_;

	return NSAccessibilityOrientationAttribute_ = new NSString(bindings.NSAccessibilityOrientationAttribute);
}

NSString NSAccessibilityVerticalOrientationValue ()
{
	if (NSAccessibilityVerticalOrientationValue_)
		return NSAccessibilityVerticalOrientationValue_;

	return NSAccessibilityVerticalOrientationValue_ = new NSString(bindings.NSAccessibilityVerticalOrientationValue);
}

NSString NSAccessibilityHorizontalOrientationValue ()
{
	if (NSAccessibilityHorizontalOrientationValue_)
		return NSAccessibilityHorizontalOrientationValue_;

	return NSAccessibilityHorizontalOrientationValue_ = new NSString(bindings.NSAccessibilityHorizontalOrientationValue);
}

NSString NSAccessibilityColumnTitlesAttribute ()
{
	if (NSAccessibilityColumnTitlesAttribute_)
		return NSAccessibilityColumnTitlesAttribute_;

	return NSAccessibilityColumnTitlesAttribute_ = new NSString(bindings.NSAccessibilityColumnTitlesAttribute);
}

NSString NSAccessibilitySearchButtonAttribute ()
{
	if (NSAccessibilitySearchButtonAttribute_)
		return NSAccessibilitySearchButtonAttribute_;

	return NSAccessibilitySearchButtonAttribute_ = new NSString(bindings.NSAccessibilitySearchButtonAttribute);
}

NSString NSAccessibilitySearchMenuAttribute ()
{
	if (NSAccessibilitySearchMenuAttribute_)
		return NSAccessibilitySearchMenuAttribute_;

	return NSAccessibilitySearchMenuAttribute_ = new NSString(bindings.NSAccessibilitySearchMenuAttribute);
}

NSString NSAccessibilityClearButtonAttribute ()
{
	if (NSAccessibilityClearButtonAttribute_)
		return NSAccessibilityClearButtonAttribute_;

	return NSAccessibilityClearButtonAttribute_ = new NSString(bindings.NSAccessibilityClearButtonAttribute);
}

NSString NSAccessibilityRowsAttribute ()
{
	if (NSAccessibilityRowsAttribute_)
		return NSAccessibilityRowsAttribute_;

	return NSAccessibilityRowsAttribute_ = new NSString(bindings.NSAccessibilityRowsAttribute);
}

NSString NSAccessibilityVisibleRowsAttribute ()
{
	if (NSAccessibilityVisibleRowsAttribute_)
		return NSAccessibilityVisibleRowsAttribute_;

	return NSAccessibilityVisibleRowsAttribute_ = new NSString(bindings.NSAccessibilityVisibleRowsAttribute);
}

NSString NSAccessibilitySelectedRowsAttribute ()
{
	if (NSAccessibilitySelectedRowsAttribute_)
		return NSAccessibilitySelectedRowsAttribute_;

	return NSAccessibilitySelectedRowsAttribute_ = new NSString(bindings.NSAccessibilitySelectedRowsAttribute);
}

NSString NSAccessibilityColumnsAttribute ()
{
	if (NSAccessibilityColumnsAttribute_)
		return NSAccessibilityColumnsAttribute_;

	return NSAccessibilityColumnsAttribute_ = new NSString(bindings.NSAccessibilityColumnsAttribute);
}

NSString NSAccessibilityVisibleColumnsAttribute ()
{
	if (NSAccessibilityVisibleColumnsAttribute_)
		return NSAccessibilityVisibleColumnsAttribute_;

	return NSAccessibilityVisibleColumnsAttribute_ = new NSString(bindings.NSAccessibilityVisibleColumnsAttribute);
}

NSString NSAccessibilitySelectedColumnsAttribute ()
{
	if (NSAccessibilitySelectedColumnsAttribute_)
		return NSAccessibilitySelectedColumnsAttribute_;

	return NSAccessibilitySelectedColumnsAttribute_ = new NSString(bindings.NSAccessibilitySelectedColumnsAttribute);
}

NSString NSAccessibilitySortDirectionAttribute ()
{
	if (NSAccessibilitySortDirectionAttribute_)
		return NSAccessibilitySortDirectionAttribute_;

	return NSAccessibilitySortDirectionAttribute_ = new NSString(bindings.NSAccessibilitySortDirectionAttribute);
}

NSString NSAccessibilityAscendingSortDirectionValue ()
{
	if (NSAccessibilityAscendingSortDirectionValue_)
		return NSAccessibilityAscendingSortDirectionValue_;

	return NSAccessibilityAscendingSortDirectionValue_ = new NSString(bindings.NSAccessibilityAscendingSortDirectionValue);
}

NSString NSAccessibilityDescendingSortDirectionValue ()
{
	if (NSAccessibilityDescendingSortDirectionValue_)
		return NSAccessibilityDescendingSortDirectionValue_;

	return NSAccessibilityDescendingSortDirectionValue_ = new NSString(bindings.NSAccessibilityDescendingSortDirectionValue);
}

NSString NSAccessibilityUnknownSortDirectionValue ()
{
	if (NSAccessibilityUnknownSortDirectionValue_)
		return NSAccessibilityUnknownSortDirectionValue_;

	return NSAccessibilityUnknownSortDirectionValue_ = new NSString(bindings.NSAccessibilityUnknownSortDirectionValue);
}

NSString NSAccessibilityDisclosingAttribute ()
{
	if (NSAccessibilityDisclosingAttribute_)
		return NSAccessibilityDisclosingAttribute_;

	return NSAccessibilityDisclosingAttribute_ = new NSString(bindings.NSAccessibilityDisclosingAttribute);
}

NSString NSAccessibilityDisclosedRowsAttribute ()
{
	if (NSAccessibilityDisclosedRowsAttribute_)
		return NSAccessibilityDisclosedRowsAttribute_;

	return NSAccessibilityDisclosedRowsAttribute_ = new NSString(bindings.NSAccessibilityDisclosedRowsAttribute);
}

NSString NSAccessibilityDisclosedByRowAttribute ()
{
	if (NSAccessibilityDisclosedByRowAttribute_)
		return NSAccessibilityDisclosedByRowAttribute_;

	return NSAccessibilityDisclosedByRowAttribute_ = new NSString(bindings.NSAccessibilityDisclosedByRowAttribute);
}

NSString NSAccessibilityDisclosureLevelAttribute ()
{
	if (NSAccessibilityDisclosureLevelAttribute_)
		return NSAccessibilityDisclosureLevelAttribute_;

	return NSAccessibilityDisclosureLevelAttribute_ = new NSString(bindings.NSAccessibilityDisclosureLevelAttribute);
}

NSString NSAccessibilityAllowedValuesAttribute ()
{
	if (NSAccessibilityAllowedValuesAttribute_)
		return NSAccessibilityAllowedValuesAttribute_;

	return NSAccessibilityAllowedValuesAttribute_ = new NSString(bindings.NSAccessibilityAllowedValuesAttribute);
}

NSString NSAccessibilityLabelUIElementsAttribute ()
{
	if (NSAccessibilityLabelUIElementsAttribute_)
		return NSAccessibilityLabelUIElementsAttribute_;

	return NSAccessibilityLabelUIElementsAttribute_ = new NSString(bindings.NSAccessibilityLabelUIElementsAttribute);
}

NSString NSAccessibilityLabelValueAttribute ()
{
	if (NSAccessibilityLabelValueAttribute_)
		return NSAccessibilityLabelValueAttribute_;

	return NSAccessibilityLabelValueAttribute_ = new NSString(bindings.NSAccessibilityLabelValueAttribute);
}

NSString NSAccessibilityMatteHoleAttribute ()
{
	if (NSAccessibilityMatteHoleAttribute_)
		return NSAccessibilityMatteHoleAttribute_;

	return NSAccessibilityMatteHoleAttribute_ = new NSString(bindings.NSAccessibilityMatteHoleAttribute);
}

NSString NSAccessibilityMatteContentUIElementAttribute ()
{
	if (NSAccessibilityMatteContentUIElementAttribute_)
		return NSAccessibilityMatteContentUIElementAttribute_;

	return NSAccessibilityMatteContentUIElementAttribute_ = new NSString(bindings.NSAccessibilityMatteContentUIElementAttribute);
}

NSString NSAccessibilityMarkerUIElementsAttribute ()
{
	if (NSAccessibilityMarkerUIElementsAttribute_)
		return NSAccessibilityMarkerUIElementsAttribute_;

	return NSAccessibilityMarkerUIElementsAttribute_ = new NSString(bindings.NSAccessibilityMarkerUIElementsAttribute);
}

NSString NSAccessibilityMarkerValuesAttribute ()
{
	if (NSAccessibilityMarkerValuesAttribute_)
		return NSAccessibilityMarkerValuesAttribute_;

	return NSAccessibilityMarkerValuesAttribute_ = new NSString(bindings.NSAccessibilityMarkerValuesAttribute);
}

NSString NSAccessibilityMarkerGroupUIElementAttribute ()
{
	if (NSAccessibilityMarkerGroupUIElementAttribute_)
		return NSAccessibilityMarkerGroupUIElementAttribute_;

	return NSAccessibilityMarkerGroupUIElementAttribute_ = new NSString(bindings.NSAccessibilityMarkerGroupUIElementAttribute);
}

NSString NSAccessibilityUnitsAttribute ()
{
	if (NSAccessibilityUnitsAttribute_)
		return NSAccessibilityUnitsAttribute_;

	return NSAccessibilityUnitsAttribute_ = new NSString(bindings.NSAccessibilityUnitsAttribute);
}

NSString NSAccessibilityUnitDescriptionAttribute ()
{
	if (NSAccessibilityUnitDescriptionAttribute_)
		return NSAccessibilityUnitDescriptionAttribute_;

	return NSAccessibilityUnitDescriptionAttribute_ = new NSString(bindings.NSAccessibilityUnitDescriptionAttribute);
}

NSString NSAccessibilityMarkerTypeAttribute ()
{
	if (NSAccessibilityMarkerTypeAttribute_)
		return NSAccessibilityMarkerTypeAttribute_;

	return NSAccessibilityMarkerTypeAttribute_ = new NSString(bindings.NSAccessibilityMarkerTypeAttribute);
}

NSString NSAccessibilityMarkerTypeDescriptionAttribute ()
{
	if (NSAccessibilityMarkerTypeDescriptionAttribute_)
		return NSAccessibilityMarkerTypeDescriptionAttribute_;

	return NSAccessibilityMarkerTypeDescriptionAttribute_ = new NSString(bindings.NSAccessibilityMarkerTypeDescriptionAttribute);
}

NSString NSAccessibilityLeftTabStopMarkerTypeValue ()
{
	if (NSAccessibilityLeftTabStopMarkerTypeValue_)
		return NSAccessibilityLeftTabStopMarkerTypeValue_;

	return NSAccessibilityLeftTabStopMarkerTypeValue_ = new NSString(bindings.NSAccessibilityLeftTabStopMarkerTypeValue);
}

NSString NSAccessibilityRightTabStopMarkerTypeValue ()
{
	if (NSAccessibilityRightTabStopMarkerTypeValue_)
		return NSAccessibilityRightTabStopMarkerTypeValue_;

	return NSAccessibilityRightTabStopMarkerTypeValue_ = new NSString(bindings.NSAccessibilityRightTabStopMarkerTypeValue);
}

NSString NSAccessibilityCenterTabStopMarkerTypeValue ()
{
	if (NSAccessibilityCenterTabStopMarkerTypeValue_)
		return NSAccessibilityCenterTabStopMarkerTypeValue_;

	return NSAccessibilityCenterTabStopMarkerTypeValue_ = new NSString(bindings.NSAccessibilityCenterTabStopMarkerTypeValue);
}

NSString NSAccessibilityDecimalTabStopMarkerTypeValue ()
{
	if (NSAccessibilityDecimalTabStopMarkerTypeValue_)
		return NSAccessibilityDecimalTabStopMarkerTypeValue_;

	return NSAccessibilityDecimalTabStopMarkerTypeValue_ = new NSString(bindings.NSAccessibilityDecimalTabStopMarkerTypeValue);
}

NSString NSAccessibilityHeadIndentMarkerTypeValue ()
{
	if (NSAccessibilityHeadIndentMarkerTypeValue_)
		return NSAccessibilityHeadIndentMarkerTypeValue_;

	return NSAccessibilityHeadIndentMarkerTypeValue_ = new NSString(bindings.NSAccessibilityHeadIndentMarkerTypeValue);
}

NSString NSAccessibilityTailIndentMarkerTypeValue ()
{
	if (NSAccessibilityTailIndentMarkerTypeValue_)
		return NSAccessibilityTailIndentMarkerTypeValue_;

	return NSAccessibilityTailIndentMarkerTypeValue_ = new NSString(bindings.NSAccessibilityTailIndentMarkerTypeValue);
}

NSString NSAccessibilityFirstLineIndentMarkerTypeValue ()
{
	if (NSAccessibilityFirstLineIndentMarkerTypeValue_)
		return NSAccessibilityFirstLineIndentMarkerTypeValue_;

	return NSAccessibilityFirstLineIndentMarkerTypeValue_ = new NSString(bindings.NSAccessibilityFirstLineIndentMarkerTypeValue);
}

NSString NSAccessibilityUnknownMarkerTypeValue ()
{
	if (NSAccessibilityUnknownMarkerTypeValue_)
		return NSAccessibilityUnknownMarkerTypeValue_;

	return NSAccessibilityUnknownMarkerTypeValue_ = new NSString(bindings.NSAccessibilityUnknownMarkerTypeValue);
}

NSString NSAccessibilityInchesUnitValue ()
{
	if (NSAccessibilityInchesUnitValue_)
		return NSAccessibilityInchesUnitValue_;

	return NSAccessibilityInchesUnitValue_ = new NSString(bindings.NSAccessibilityInchesUnitValue);
}

NSString NSAccessibilityCentimetersUnitValue ()
{
	if (NSAccessibilityCentimetersUnitValue_)
		return NSAccessibilityCentimetersUnitValue_;

	return NSAccessibilityCentimetersUnitValue_ = new NSString(bindings.NSAccessibilityCentimetersUnitValue);
}

NSString NSAccessibilityPointsUnitValue ()
{
	if (NSAccessibilityPointsUnitValue_)
		return NSAccessibilityPointsUnitValue_;

	return NSAccessibilityPointsUnitValue_ = new NSString(bindings.NSAccessibilityPointsUnitValue);
}

NSString NSAccessibilityPicasUnitValue ()
{
	if (NSAccessibilityPicasUnitValue_)
		return NSAccessibilityPicasUnitValue_;

	return NSAccessibilityPicasUnitValue_ = new NSString(bindings.NSAccessibilityPicasUnitValue);
}

NSString NSAccessibilityUnknownUnitValue ()
{
	if (NSAccessibilityUnknownUnitValue_)
		return NSAccessibilityUnknownUnitValue_;

	return NSAccessibilityUnknownUnitValue_ = new NSString(bindings.NSAccessibilityUnknownUnitValue);
}

NSString NSAccessibilityPressAction ()
{
	if (NSAccessibilityPressAction_)
		return NSAccessibilityPressAction_;

	return NSAccessibilityPressAction_ = new NSString(bindings.NSAccessibilityPressAction);
}

NSString NSAccessibilityIncrementAction ()
{
	if (NSAccessibilityIncrementAction_)
		return NSAccessibilityIncrementAction_;

	return NSAccessibilityIncrementAction_ = new NSString(bindings.NSAccessibilityIncrementAction);
}

NSString NSAccessibilityDecrementAction ()
{
	if (NSAccessibilityDecrementAction_)
		return NSAccessibilityDecrementAction_;

	return NSAccessibilityDecrementAction_ = new NSString(bindings.NSAccessibilityDecrementAction);
}

NSString NSAccessibilityConfirmAction ()
{
	if (NSAccessibilityConfirmAction_)
		return NSAccessibilityConfirmAction_;

	return NSAccessibilityConfirmAction_ = new NSString(bindings.NSAccessibilityConfirmAction);
}

NSString NSAccessibilityPickAction ()
{
	if (NSAccessibilityPickAction_)
		return NSAccessibilityPickAction_;

	return NSAccessibilityPickAction_ = new NSString(bindings.NSAccessibilityPickAction);
}

NSString NSAccessibilityCancelAction ()
{
	if (NSAccessibilityCancelAction_)
		return NSAccessibilityCancelAction_;

	return NSAccessibilityCancelAction_ = new NSString(bindings.NSAccessibilityCancelAction);
}

NSString NSAccessibilityRaiseAction ()
{
	if (NSAccessibilityRaiseAction_)
		return NSAccessibilityRaiseAction_;

	return NSAccessibilityRaiseAction_ = new NSString(bindings.NSAccessibilityRaiseAction);
}

NSString NSAccessibilityShowMenuAction ()
{
	if (NSAccessibilityShowMenuAction_)
		return NSAccessibilityShowMenuAction_;

	return NSAccessibilityShowMenuAction_ = new NSString(bindings.NSAccessibilityShowMenuAction);
}

NSString NSAccessibilityDeleteAction ()
{
	if (NSAccessibilityDeleteAction_)
		return NSAccessibilityDeleteAction_;

	return NSAccessibilityDeleteAction_ = new NSString(bindings.NSAccessibilityDeleteAction);
}

NSString NSAccessibilityMainWindowChangedNotification ()
{
	if (NSAccessibilityMainWindowChangedNotification_)
		return NSAccessibilityMainWindowChangedNotification_;

	return NSAccessibilityMainWindowChangedNotification_ = new NSString(bindings.NSAccessibilityMainWindowChangedNotification);
}

NSString NSAccessibilityFocusedWindowChangedNotification ()
{
	if (NSAccessibilityFocusedWindowChangedNotification_)
		return NSAccessibilityFocusedWindowChangedNotification_;

	return NSAccessibilityFocusedWindowChangedNotification_ = new NSString(bindings.NSAccessibilityFocusedWindowChangedNotification);
}

NSString NSAccessibilityFocusedUIElementChangedNotification ()
{
	if (NSAccessibilityFocusedUIElementChangedNotification_)
		return NSAccessibilityFocusedUIElementChangedNotification_;

	return NSAccessibilityFocusedUIElementChangedNotification_ = new NSString(bindings.NSAccessibilityFocusedUIElementChangedNotification);
}

NSString NSAccessibilityApplicationActivatedNotification ()
{
	if (NSAccessibilityApplicationActivatedNotification_)
		return NSAccessibilityApplicationActivatedNotification_;

	return NSAccessibilityApplicationActivatedNotification_ = new NSString(bindings.NSAccessibilityApplicationActivatedNotification);
}

NSString NSAccessibilityApplicationDeactivatedNotification ()
{
	if (NSAccessibilityApplicationDeactivatedNotification_)
		return NSAccessibilityApplicationDeactivatedNotification_;

	return NSAccessibilityApplicationDeactivatedNotification_ = new NSString(bindings.NSAccessibilityApplicationDeactivatedNotification);
}

NSString NSAccessibilityApplicationHiddenNotification ()
{
	if (NSAccessibilityApplicationHiddenNotification_)
		return NSAccessibilityApplicationHiddenNotification_;

	return NSAccessibilityApplicationHiddenNotification_ = new NSString(bindings.NSAccessibilityApplicationHiddenNotification);
}

NSString NSAccessibilityApplicationShownNotification ()
{
	if (NSAccessibilityApplicationShownNotification_)
		return NSAccessibilityApplicationShownNotification_;

	return NSAccessibilityApplicationShownNotification_ = new NSString(bindings.NSAccessibilityApplicationShownNotification);
}

NSString NSAccessibilityWindowCreatedNotification ()
{
	if (NSAccessibilityWindowCreatedNotification_)
		return NSAccessibilityWindowCreatedNotification_;

	return NSAccessibilityWindowCreatedNotification_ = new NSString(bindings.NSAccessibilityWindowCreatedNotification);
}

NSString NSAccessibilityWindowMovedNotification ()
{
	if (NSAccessibilityWindowMovedNotification_)
		return NSAccessibilityWindowMovedNotification_;

	return NSAccessibilityWindowMovedNotification_ = new NSString(bindings.NSAccessibilityWindowMovedNotification);
}

NSString NSAccessibilityWindowResizedNotification ()
{
	if (NSAccessibilityWindowResizedNotification_)
		return NSAccessibilityWindowResizedNotification_;

	return NSAccessibilityWindowResizedNotification_ = new NSString(bindings.NSAccessibilityWindowResizedNotification);
}

NSString NSAccessibilityWindowMiniaturizedNotification ()
{
	if (NSAccessibilityWindowMiniaturizedNotification_)
		return NSAccessibilityWindowMiniaturizedNotification_;

	return NSAccessibilityWindowMiniaturizedNotification_ = new NSString(bindings.NSAccessibilityWindowMiniaturizedNotification);
}

NSString NSAccessibilityWindowDeminiaturizedNotification ()
{
	if (NSAccessibilityWindowDeminiaturizedNotification_)
		return NSAccessibilityWindowDeminiaturizedNotification_;

	return NSAccessibilityWindowDeminiaturizedNotification_ = new NSString(bindings.NSAccessibilityWindowDeminiaturizedNotification);
}

NSString NSAccessibilityDrawerCreatedNotification ()
{
	if (NSAccessibilityDrawerCreatedNotification_)
		return NSAccessibilityDrawerCreatedNotification_;

	return NSAccessibilityDrawerCreatedNotification_ = new NSString(bindings.NSAccessibilityDrawerCreatedNotification);
}

NSString NSAccessibilitySheetCreatedNotification ()
{
	if (NSAccessibilitySheetCreatedNotification_)
		return NSAccessibilitySheetCreatedNotification_;

	return NSAccessibilitySheetCreatedNotification_ = new NSString(bindings.NSAccessibilitySheetCreatedNotification);
}

NSString NSAccessibilityUIElementDestroyedNotification ()
{
	if (NSAccessibilityUIElementDestroyedNotification_)
		return NSAccessibilityUIElementDestroyedNotification_;

	return NSAccessibilityUIElementDestroyedNotification_ = new NSString(bindings.NSAccessibilityUIElementDestroyedNotification);
}

NSString NSAccessibilityValueChangedNotification ()
{
	if (NSAccessibilityValueChangedNotification_)
		return NSAccessibilityValueChangedNotification_;

	return NSAccessibilityValueChangedNotification_ = new NSString(bindings.NSAccessibilityValueChangedNotification);
}

NSString NSAccessibilityTitleChangedNotification ()
{
	if (NSAccessibilityTitleChangedNotification_)
		return NSAccessibilityTitleChangedNotification_;

	return NSAccessibilityTitleChangedNotification_ = new NSString(bindings.NSAccessibilityTitleChangedNotification);
}

NSString NSAccessibilityResizedNotification ()
{
	if (NSAccessibilityResizedNotification_)
		return NSAccessibilityResizedNotification_;

	return NSAccessibilityResizedNotification_ = new NSString(bindings.NSAccessibilityResizedNotification);
}

NSString NSAccessibilityMovedNotification ()
{
	if (NSAccessibilityMovedNotification_)
		return NSAccessibilityMovedNotification_;

	return NSAccessibilityMovedNotification_ = new NSString(bindings.NSAccessibilityMovedNotification);
}

NSString NSAccessibilityCreatedNotification ()
{
	if (NSAccessibilityCreatedNotification_)
		return NSAccessibilityCreatedNotification_;

	return NSAccessibilityCreatedNotification_ = new NSString(bindings.NSAccessibilityCreatedNotification);
}

NSString NSAccessibilityHelpTagCreatedNotification ()
{
	if (NSAccessibilityHelpTagCreatedNotification_)
		return NSAccessibilityHelpTagCreatedNotification_;

	return NSAccessibilityHelpTagCreatedNotification_ = new NSString(bindings.NSAccessibilityHelpTagCreatedNotification);
}

NSString NSAccessibilitySelectedTextChangedNotification ()
{
	if (NSAccessibilitySelectedTextChangedNotification_)
		return NSAccessibilitySelectedTextChangedNotification_;

	return NSAccessibilitySelectedTextChangedNotification_ = new NSString(bindings.NSAccessibilitySelectedTextChangedNotification);
}

NSString NSAccessibilityRowCountChangedNotification ()
{
	if (NSAccessibilityRowCountChangedNotification_)
		return NSAccessibilityRowCountChangedNotification_;

	return NSAccessibilityRowCountChangedNotification_ = new NSString(bindings.NSAccessibilityRowCountChangedNotification);
}

NSString NSAccessibilitySelectedChildrenChangedNotification ()
{
	if (NSAccessibilitySelectedChildrenChangedNotification_)
		return NSAccessibilitySelectedChildrenChangedNotification_;

	return NSAccessibilitySelectedChildrenChangedNotification_ = new NSString(bindings.NSAccessibilitySelectedChildrenChangedNotification);
}

NSString NSAccessibilitySelectedRowsChangedNotification ()
{
	if (NSAccessibilitySelectedRowsChangedNotification_)
		return NSAccessibilitySelectedRowsChangedNotification_;

	return NSAccessibilitySelectedRowsChangedNotification_ = new NSString(bindings.NSAccessibilitySelectedRowsChangedNotification);
}

NSString NSAccessibilitySelectedColumnsChangedNotification ()
{
	if (NSAccessibilitySelectedColumnsChangedNotification_)
		return NSAccessibilitySelectedColumnsChangedNotification_;

	return NSAccessibilitySelectedColumnsChangedNotification_ = new NSString(bindings.NSAccessibilitySelectedColumnsChangedNotification);
}

NSString NSAccessibilityUnknownRole ()
{
	if (NSAccessibilityUnknownRole_)
		return NSAccessibilityUnknownRole_;

	return NSAccessibilityUnknownRole_ = new NSString(bindings.NSAccessibilityUnknownRole);
}

NSString NSAccessibilityButtonRole ()
{
	if (NSAccessibilityButtonRole_)
		return NSAccessibilityButtonRole_;

	return NSAccessibilityButtonRole_ = new NSString(bindings.NSAccessibilityButtonRole);
}

NSString NSAccessibilityRadioButtonRole ()
{
	if (NSAccessibilityRadioButtonRole_)
		return NSAccessibilityRadioButtonRole_;

	return NSAccessibilityRadioButtonRole_ = new NSString(bindings.NSAccessibilityRadioButtonRole);
}

NSString NSAccessibilityCheckBoxRole ()
{
	if (NSAccessibilityCheckBoxRole_)
		return NSAccessibilityCheckBoxRole_;

	return NSAccessibilityCheckBoxRole_ = new NSString(bindings.NSAccessibilityCheckBoxRole);
}

NSString NSAccessibilitySliderRole ()
{
	if (NSAccessibilitySliderRole_)
		return NSAccessibilitySliderRole_;

	return NSAccessibilitySliderRole_ = new NSString(bindings.NSAccessibilitySliderRole);
}

NSString NSAccessibilityTabGroupRole ()
{
	if (NSAccessibilityTabGroupRole_)
		return NSAccessibilityTabGroupRole_;

	return NSAccessibilityTabGroupRole_ = new NSString(bindings.NSAccessibilityTabGroupRole);
}

NSString NSAccessibilityTextFieldRole ()
{
	if (NSAccessibilityTextFieldRole_)
		return NSAccessibilityTextFieldRole_;

	return NSAccessibilityTextFieldRole_ = new NSString(bindings.NSAccessibilityTextFieldRole);
}

NSString NSAccessibilityStaticTextRole ()
{
	if (NSAccessibilityStaticTextRole_)
		return NSAccessibilityStaticTextRole_;

	return NSAccessibilityStaticTextRole_ = new NSString(bindings.NSAccessibilityStaticTextRole);
}

NSString NSAccessibilityTextAreaRole ()
{
	if (NSAccessibilityTextAreaRole_)
		return NSAccessibilityTextAreaRole_;

	return NSAccessibilityTextAreaRole_ = new NSString(bindings.NSAccessibilityTextAreaRole);
}

NSString NSAccessibilityScrollAreaRole ()
{
	if (NSAccessibilityScrollAreaRole_)
		return NSAccessibilityScrollAreaRole_;

	return NSAccessibilityScrollAreaRole_ = new NSString(bindings.NSAccessibilityScrollAreaRole);
}

NSString NSAccessibilityPopUpButtonRole ()
{
	if (NSAccessibilityPopUpButtonRole_)
		return NSAccessibilityPopUpButtonRole_;

	return NSAccessibilityPopUpButtonRole_ = new NSString(bindings.NSAccessibilityPopUpButtonRole);
}

NSString NSAccessibilityMenuButtonRole ()
{
	if (NSAccessibilityMenuButtonRole_)
		return NSAccessibilityMenuButtonRole_;

	return NSAccessibilityMenuButtonRole_ = new NSString(bindings.NSAccessibilityMenuButtonRole);
}

NSString NSAccessibilityTableRole ()
{
	if (NSAccessibilityTableRole_)
		return NSAccessibilityTableRole_;

	return NSAccessibilityTableRole_ = new NSString(bindings.NSAccessibilityTableRole);
}

NSString NSAccessibilityApplicationRole ()
{
	if (NSAccessibilityApplicationRole_)
		return NSAccessibilityApplicationRole_;

	return NSAccessibilityApplicationRole_ = new NSString(bindings.NSAccessibilityApplicationRole);
}

NSString NSAccessibilityGroupRole ()
{
	if (NSAccessibilityGroupRole_)
		return NSAccessibilityGroupRole_;

	return NSAccessibilityGroupRole_ = new NSString(bindings.NSAccessibilityGroupRole);
}

NSString NSAccessibilityRadioGroupRole ()
{
	if (NSAccessibilityRadioGroupRole_)
		return NSAccessibilityRadioGroupRole_;

	return NSAccessibilityRadioGroupRole_ = new NSString(bindings.NSAccessibilityRadioGroupRole);
}

NSString NSAccessibilityListRole ()
{
	if (NSAccessibilityListRole_)
		return NSAccessibilityListRole_;

	return NSAccessibilityListRole_ = new NSString(bindings.NSAccessibilityListRole);
}

NSString NSAccessibilityScrollBarRole ()
{
	if (NSAccessibilityScrollBarRole_)
		return NSAccessibilityScrollBarRole_;

	return NSAccessibilityScrollBarRole_ = new NSString(bindings.NSAccessibilityScrollBarRole);
}

NSString NSAccessibilityValueIndicatorRole ()
{
	if (NSAccessibilityValueIndicatorRole_)
		return NSAccessibilityValueIndicatorRole_;

	return NSAccessibilityValueIndicatorRole_ = new NSString(bindings.NSAccessibilityValueIndicatorRole);
}

NSString NSAccessibilityImageRole ()
{
	if (NSAccessibilityImageRole_)
		return NSAccessibilityImageRole_;

	return NSAccessibilityImageRole_ = new NSString(bindings.NSAccessibilityImageRole);
}

NSString NSAccessibilityMenuBarRole ()
{
	if (NSAccessibilityMenuBarRole_)
		return NSAccessibilityMenuBarRole_;

	return NSAccessibilityMenuBarRole_ = new NSString(bindings.NSAccessibilityMenuBarRole);
}

NSString NSAccessibilityMenuRole ()
{
	if (NSAccessibilityMenuRole_)
		return NSAccessibilityMenuRole_;

	return NSAccessibilityMenuRole_ = new NSString(bindings.NSAccessibilityMenuRole);
}

NSString NSAccessibilityMenuItemRole ()
{
	if (NSAccessibilityMenuItemRole_)
		return NSAccessibilityMenuItemRole_;

	return NSAccessibilityMenuItemRole_ = new NSString(bindings.NSAccessibilityMenuItemRole);
}

NSString NSAccessibilityColumnRole ()
{
	if (NSAccessibilityColumnRole_)
		return NSAccessibilityColumnRole_;

	return NSAccessibilityColumnRole_ = new NSString(bindings.NSAccessibilityColumnRole);
}

NSString NSAccessibilityRowRole ()
{
	if (NSAccessibilityRowRole_)
		return NSAccessibilityRowRole_;

	return NSAccessibilityRowRole_ = new NSString(bindings.NSAccessibilityRowRole);
}

NSString NSAccessibilityToolbarRole ()
{
	if (NSAccessibilityToolbarRole_)
		return NSAccessibilityToolbarRole_;

	return NSAccessibilityToolbarRole_ = new NSString(bindings.NSAccessibilityToolbarRole);
}

NSString NSAccessibilityBusyIndicatorRole ()
{
	if (NSAccessibilityBusyIndicatorRole_)
		return NSAccessibilityBusyIndicatorRole_;

	return NSAccessibilityBusyIndicatorRole_ = new NSString(bindings.NSAccessibilityBusyIndicatorRole);
}

NSString NSAccessibilityProgressIndicatorRole ()
{
	if (NSAccessibilityProgressIndicatorRole_)
		return NSAccessibilityProgressIndicatorRole_;

	return NSAccessibilityProgressIndicatorRole_ = new NSString(bindings.NSAccessibilityProgressIndicatorRole);
}

NSString NSAccessibilityWindowRole ()
{
	if (NSAccessibilityWindowRole_)
		return NSAccessibilityWindowRole_;

	return NSAccessibilityWindowRole_ = new NSString(bindings.NSAccessibilityWindowRole);
}

NSString NSAccessibilityDrawerRole ()
{
	if (NSAccessibilityDrawerRole_)
		return NSAccessibilityDrawerRole_;

	return NSAccessibilityDrawerRole_ = new NSString(bindings.NSAccessibilityDrawerRole);
}

NSString NSAccessibilitySystemWideRole ()
{
	if (NSAccessibilitySystemWideRole_)
		return NSAccessibilitySystemWideRole_;

	return NSAccessibilitySystemWideRole_ = new NSString(bindings.NSAccessibilitySystemWideRole);
}

NSString NSAccessibilityOutlineRole ()
{
	if (NSAccessibilityOutlineRole_)
		return NSAccessibilityOutlineRole_;

	return NSAccessibilityOutlineRole_ = new NSString(bindings.NSAccessibilityOutlineRole);
}

NSString NSAccessibilityIncrementorRole ()
{
	if (NSAccessibilityIncrementorRole_)
		return NSAccessibilityIncrementorRole_;

	return NSAccessibilityIncrementorRole_ = new NSString(bindings.NSAccessibilityIncrementorRole);
}

NSString NSAccessibilityBrowserRole ()
{
	if (NSAccessibilityBrowserRole_)
		return NSAccessibilityBrowserRole_;

	return NSAccessibilityBrowserRole_ = new NSString(bindings.NSAccessibilityBrowserRole);
}

NSString NSAccessibilityComboBoxRole ()
{
	if (NSAccessibilityComboBoxRole_)
		return NSAccessibilityComboBoxRole_;

	return NSAccessibilityComboBoxRole_ = new NSString(bindings.NSAccessibilityComboBoxRole);
}

NSString NSAccessibilitySplitGroupRole ()
{
	if (NSAccessibilitySplitGroupRole_)
		return NSAccessibilitySplitGroupRole_;

	return NSAccessibilitySplitGroupRole_ = new NSString(bindings.NSAccessibilitySplitGroupRole);
}

NSString NSAccessibilitySplitterRole ()
{
	if (NSAccessibilitySplitterRole_)
		return NSAccessibilitySplitterRole_;

	return NSAccessibilitySplitterRole_ = new NSString(bindings.NSAccessibilitySplitterRole);
}

NSString NSAccessibilityColorWellRole ()
{
	if (NSAccessibilityColorWellRole_)
		return NSAccessibilityColorWellRole_;

	return NSAccessibilityColorWellRole_ = new NSString(bindings.NSAccessibilityColorWellRole);
}

NSString NSAccessibilityGrowAreaRole ()
{
	if (NSAccessibilityGrowAreaRole_)
		return NSAccessibilityGrowAreaRole_;

	return NSAccessibilityGrowAreaRole_ = new NSString(bindings.NSAccessibilityGrowAreaRole);
}

NSString NSAccessibilitySheetRole ()
{
	if (NSAccessibilitySheetRole_)
		return NSAccessibilitySheetRole_;

	return NSAccessibilitySheetRole_ = new NSString(bindings.NSAccessibilitySheetRole);
}

NSString NSAccessibilityHelpTagRole ()
{
	if (NSAccessibilityHelpTagRole_)
		return NSAccessibilityHelpTagRole_;

	return NSAccessibilityHelpTagRole_ = new NSString(bindings.NSAccessibilityHelpTagRole);
}

NSString NSAccessibilityMatteRole ()
{
	if (NSAccessibilityMatteRole_)
		return NSAccessibilityMatteRole_;

	return NSAccessibilityMatteRole_ = new NSString(bindings.NSAccessibilityMatteRole);
}

NSString NSAccessibilityRulerRole ()
{
	if (NSAccessibilityRulerRole_)
		return NSAccessibilityRulerRole_;

	return NSAccessibilityRulerRole_ = new NSString(bindings.NSAccessibilityRulerRole);
}

NSString NSAccessibilityRulerMarkerRole ()
{
	if (NSAccessibilityRulerMarkerRole_)
		return NSAccessibilityRulerMarkerRole_;

	return NSAccessibilityRulerMarkerRole_ = new NSString(bindings.NSAccessibilityRulerMarkerRole);
}

NSString NSAccessibilitySortButtonRole ()
{
	if (NSAccessibilitySortButtonRole_)
		return NSAccessibilitySortButtonRole_;

	return NSAccessibilitySortButtonRole_ = new NSString(bindings.NSAccessibilitySortButtonRole);
}

NSString NSAccessibilityLinkRole ()
{
	if (NSAccessibilityLinkRole_)
		return NSAccessibilityLinkRole_;

	return NSAccessibilityLinkRole_ = new NSString(bindings.NSAccessibilityLinkRole);
}

NSString NSAccessibilityDisclosureTriangleRole ()
{
	if (NSAccessibilityDisclosureTriangleRole_)
		return NSAccessibilityDisclosureTriangleRole_;

	return NSAccessibilityDisclosureTriangleRole_ = new NSString(bindings.NSAccessibilityDisclosureTriangleRole);
}

NSString NSAccessibilityGridRole ()
{
	if (NSAccessibilityGridRole_)
		return NSAccessibilityGridRole_;

	return NSAccessibilityGridRole_ = new NSString(bindings.NSAccessibilityGridRole);
}

NSString NSAccessibilityUnknownSubrole ()
{
	if (NSAccessibilityUnknownSubrole_)
		return NSAccessibilityUnknownSubrole_;

	return NSAccessibilityUnknownSubrole_ = new NSString(bindings.NSAccessibilityUnknownSubrole);
}

NSString NSAccessibilityCloseButtonSubrole ()
{
	if (NSAccessibilityCloseButtonSubrole_)
		return NSAccessibilityCloseButtonSubrole_;

	return NSAccessibilityCloseButtonSubrole_ = new NSString(bindings.NSAccessibilityCloseButtonSubrole);
}

NSString NSAccessibilityZoomButtonSubrole ()
{
	if (NSAccessibilityZoomButtonSubrole_)
		return NSAccessibilityZoomButtonSubrole_;

	return NSAccessibilityZoomButtonSubrole_ = new NSString(bindings.NSAccessibilityZoomButtonSubrole);
}

NSString NSAccessibilityMinimizeButtonSubrole ()
{
	if (NSAccessibilityMinimizeButtonSubrole_)
		return NSAccessibilityMinimizeButtonSubrole_;

	return NSAccessibilityMinimizeButtonSubrole_ = new NSString(bindings.NSAccessibilityMinimizeButtonSubrole);
}

NSString NSAccessibilityToolbarButtonSubrole ()
{
	if (NSAccessibilityToolbarButtonSubrole_)
		return NSAccessibilityToolbarButtonSubrole_;

	return NSAccessibilityToolbarButtonSubrole_ = new NSString(bindings.NSAccessibilityToolbarButtonSubrole);
}

NSString NSAccessibilityTableRowSubrole ()
{
	if (NSAccessibilityTableRowSubrole_)
		return NSAccessibilityTableRowSubrole_;

	return NSAccessibilityTableRowSubrole_ = new NSString(bindings.NSAccessibilityTableRowSubrole);
}

NSString NSAccessibilityOutlineRowSubrole ()
{
	if (NSAccessibilityOutlineRowSubrole_)
		return NSAccessibilityOutlineRowSubrole_;

	return NSAccessibilityOutlineRowSubrole_ = new NSString(bindings.NSAccessibilityOutlineRowSubrole);
}

NSString NSAccessibilitySecureTextFieldSubrole ()
{
	if (NSAccessibilitySecureTextFieldSubrole_)
		return NSAccessibilitySecureTextFieldSubrole_;

	return NSAccessibilitySecureTextFieldSubrole_ = new NSString(bindings.NSAccessibilitySecureTextFieldSubrole);
}

NSString NSAccessibilityStandardWindowSubrole ()
{
	if (NSAccessibilityStandardWindowSubrole_)
		return NSAccessibilityStandardWindowSubrole_;

	return NSAccessibilityStandardWindowSubrole_ = new NSString(bindings.NSAccessibilityStandardWindowSubrole);
}

NSString NSAccessibilityDialogSubrole ()
{
	if (NSAccessibilityDialogSubrole_)
		return NSAccessibilityDialogSubrole_;

	return NSAccessibilityDialogSubrole_ = new NSString(bindings.NSAccessibilityDialogSubrole);
}

NSString NSAccessibilitySystemDialogSubrole ()
{
	if (NSAccessibilitySystemDialogSubrole_)
		return NSAccessibilitySystemDialogSubrole_;

	return NSAccessibilitySystemDialogSubrole_ = new NSString(bindings.NSAccessibilitySystemDialogSubrole);
}

NSString NSAccessibilityFloatingWindowSubrole ()
{
	if (NSAccessibilityFloatingWindowSubrole_)
		return NSAccessibilityFloatingWindowSubrole_;

	return NSAccessibilityFloatingWindowSubrole_ = new NSString(bindings.NSAccessibilityFloatingWindowSubrole);
}

NSString NSAccessibilitySystemFloatingWindowSubrole ()
{
	if (NSAccessibilitySystemFloatingWindowSubrole_)
		return NSAccessibilitySystemFloatingWindowSubrole_;

	return NSAccessibilitySystemFloatingWindowSubrole_ = new NSString(bindings.NSAccessibilitySystemFloatingWindowSubrole);
}

NSString NSAccessibilityIncrementArrowSubrole ()
{
	if (NSAccessibilityIncrementArrowSubrole_)
		return NSAccessibilityIncrementArrowSubrole_;

	return NSAccessibilityIncrementArrowSubrole_ = new NSString(bindings.NSAccessibilityIncrementArrowSubrole);
}

NSString NSAccessibilityDecrementArrowSubrole ()
{
	if (NSAccessibilityDecrementArrowSubrole_)
		return NSAccessibilityDecrementArrowSubrole_;

	return NSAccessibilityDecrementArrowSubrole_ = new NSString(bindings.NSAccessibilityDecrementArrowSubrole);
}

NSString NSAccessibilityIncrementPageSubrole ()
{
	if (NSAccessibilityIncrementPageSubrole_)
		return NSAccessibilityIncrementPageSubrole_;

	return NSAccessibilityIncrementPageSubrole_ = new NSString(bindings.NSAccessibilityIncrementPageSubrole);
}

NSString NSAccessibilityDecrementPageSubrole ()
{
	if (NSAccessibilityDecrementPageSubrole_)
		return NSAccessibilityDecrementPageSubrole_;

	return NSAccessibilityDecrementPageSubrole_ = new NSString(bindings.NSAccessibilityDecrementPageSubrole);
}

NSString NSAccessibilitySearchFieldSubrole ()
{
	if (NSAccessibilitySearchFieldSubrole_)
		return NSAccessibilitySearchFieldSubrole_;

	return NSAccessibilitySearchFieldSubrole_ = new NSString(bindings.NSAccessibilitySearchFieldSubrole);
}

NSString NSAccessibilityTextAttachmentSubrole ()
{
	if (NSAccessibilityTextAttachmentSubrole_)
		return NSAccessibilityTextAttachmentSubrole_;

	return NSAccessibilityTextAttachmentSubrole_ = new NSString(bindings.NSAccessibilityTextAttachmentSubrole);
}

NSString NSAccessibilityTextLinkSubrole ()
{
	if (NSAccessibilityTextLinkSubrole_)
		return NSAccessibilityTextLinkSubrole_;

	return NSAccessibilityTextLinkSubrole_ = new NSString(bindings.NSAccessibilityTextLinkSubrole);
}

NSString NSAccessibilityTimelineSubrole ()
{
	if (NSAccessibilityTimelineSubrole_)
		return NSAccessibilityTimelineSubrole_;

	return NSAccessibilityTimelineSubrole_ = new NSString(bindings.NSAccessibilityTimelineSubrole);
}

const TNSAccessibility = `

	NSArray accessibilityAttributeNames ()
	{
		return invokeObjcSelf!(NSArray, "accessibilityAttributeNames");
	}

	Object accessibilityAttributeValue (NSString attribute)
	{
		return invokeObjcSelf!(Object, "accessibilityAttributeValue:", NSString)(attribute);
	}

	bool accessibilityIsAttributeSettable (NSString attribute)
	{
		return invokeObjcSelf!(bool, "accessibilityIsAttributeSettable:", NSString)(attribute);
	}

	void accessibilitySetValue (Object value, NSString attribute)
	{
		return invokeObjcSelf!(void, "accessibilitySetValue:forAttribute:", Object, NSString)(value, attribute);
	}

	NSArray accessibilityParameterizedAttributeNames ()
	{
		return invokeObjcSelf!(NSArray, "accessibilityParameterizedAttributeNames");
	}

	Object accessibilityAttributeValue (NSString attribute, Object parameter)
	{
		return invokeObjcSelf!(Object, "accessibilityAttributeValue:forParameter:", NSString, Object)(attribute, parameter);
	}

	NSArray accessibilityActionNames ()
	{
		return invokeObjcSelf!(NSArray, "accessibilityActionNames");
	}

	NSString accessibilityActionDescription (NSString action)
	{
		return invokeObjcSelf!(NSString, "accessibilityActionDescription:", NSString)(action);
	}

	void accessibilityPerformAction (NSString action)
	{
		return invokeObjcSelf!(void, "accessibilityPerformAction:", NSString)(action);
	}

	bool accessibilityIsIgnored ()
	{
		return invokeObjcSelf!(bool, "accessibilityIsIgnored");
	}

	Object accessibilityHitTest (NSPoint point)
	{
		return invokeObjcSelf!(Object, "accessibilityHitTest:", NSPoint)(point);
	}

	Object accessibilityFocusedUIElement ()
	{
		return invokeObjcSelf!(Object, "accessibilityFocusedUIElement");
	}

	//mixin ObjcBindMethod!(accessibilityAttributeNames, "accessibilityAttributeNames");
	//mixin ObjcBindMethod!(accessibilityAttributeValue, "accessibilityAttributeValue:");
	//mixin ObjcBindMethod!(accessibilityIsAttributeSettable, "accessibilityIsAttributeSettable:");
	//mixin ObjcBindMethod!(accessibilitySetValue, "accessibilitySetValue:forAttribute:");
	//mixin ObjcBindMethod!(accessibilityParameterizedAttributeNames, "accessibilityParameterizedAttributeNames");
	//mixin ObjcBindMethod!(accessibilityAttributeValue, "accessibilityAttributeValue:forParameter:");
	//mixin ObjcBindMethod!(accessibilityActionNames, "accessibilityActionNames");
	//mixin ObjcBindMethod!(accessibilityActionDescription, "accessibilityActionDescription:");
	//mixin ObjcBindMethod!(accessibilityPerformAction, "accessibilityPerformAction:");
	//mixin ObjcBindMethod!(accessibilityIsIgnored, "accessibilityIsIgnored");
	//mixin ObjcBindMethod!(accessibilityHitTest, "accessibilityHitTest:");
	//mixin ObjcBindMethod!(accessibilityFocusedUIElement, "accessibilityFocusedUIElement");

`;

const TNSAccessibilityAdditions = `

	bool accessibilitySetOverrideValue (Object value, NSString attribute)
	{
		return invokeObjcSelf!(bool, "accessibilitySetOverrideValue:forAttribute:", Object, NSString)(value, attribute);
	}

	//mixin ObjcBindMethod!(accessibilitySetOverrideValue, "accessibilitySetOverrideValue:forAttribute:");

`;
/*
NSString NSAccessibilityRoleDescription (NSString role, NSString subrole)
{
	return Bridge.invokeObjcFunction!(NSString, bindings.NSAccessibilityRoleDescription, NSString, NSString)(role, subrole);
}

NSString NSAccessibilityRoleDescriptionForUIElement (Object element)
{
	return Bridge.invokeObjcFunction!(NSString, bindings.NSAccessibilityRoleDescriptionForUIElement, Object)(element);
}

NSString NSAccessibilityActionDescription (NSString action)
{
	return Bridge.invokeObjcFunction!(NSString, bindings.NSAccessibilityActionDescription, NSString)(action);
}

void NSAccessibilityRaiseBadArgumentException (Object element, NSString attribute, Object value)
{
	return Bridge.invokeObjcFunction!(void, bindings.NSAccessibilityRaiseBadArgumentException, Object, NSString, Object)(element, attribute, value);
}

Object NSAccessibilityUnignoredAncestor (Object element)
{
	return Bridge.invokeObjcFunction!(Object, bindings.NSAccessibilityUnignoredAncestor, Object)(element);
}

Object NSAccessibilityUnignoredDescendant (Object element)
{
	return Bridge.invokeObjcFunction!(Object, bindings.NSAccessibilityUnignoredDescendant, Object)(element);
}

NSArray NSAccessibilityUnignoredChildren (NSArray originalChildren)
{
	return Bridge.invokeObjcFunction!(NSArray, bindings.NSAccessibilityUnignoredChildren, NSArray)(originalChildren);
}

NSArray NSAccessibilityUnignoredChildrenForOnlyChild (Object originalChild)
{
	return Bridge.invokeObjcFunction!(NSArray, bindings.NSAccessibilityUnignoredChildrenForOnlyChild, Object)(originalChild);
}

void NSAccessibilityPostNotification (Object element, NSString notification)
{
	return Bridge.invokeObjcFunction!(void, bindings.NSAccessibilityPostNotification, Object, NSString)(element, notification);
}

*/