/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Sep 24, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.appkit.NSTreeController;

import dstep.appkit.NSObjectController;
import dstep.appkit.NSTreeNode;
import dstep.foundation.NSArray;
import dstep.foundation.NSIndexPath;
import dstep.foundation.NSString;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSTreeController : NSObjectController
{
	mixin (ObjcWrap);

	void rearrangeObjects ()
	{
		return invokeObjcSelf!(void, "rearrangeObjects");
	}

	Object arrangedObjects ()
	{
		return invokeObjcSelf!(Object, "arrangedObjects");
	}

	void setChildrenKeyPath (NSString keyPath)
	{
		return invokeObjcSelf!(void, "setChildrenKeyPath:", NSString)(keyPath);
	}

	NSString childrenKeyPath ()
	{
		return invokeObjcSelf!(NSString, "childrenKeyPath");
	}

	void setCountKeyPath (NSString keyPath)
	{
		return invokeObjcSelf!(void, "setCountKeyPath:", NSString)(keyPath);
	}

	NSString countKeyPath ()
	{
		return invokeObjcSelf!(NSString, "countKeyPath");
	}

	void setLeafKeyPath (NSString keyPath)
	{
		return invokeObjcSelf!(void, "setLeafKeyPath:", NSString)(keyPath);
	}

	NSString leafKeyPath ()
	{
		return invokeObjcSelf!(NSString, "leafKeyPath");
	}

	void setSortDescriptors (NSArray sortDescriptors)
	{
		return invokeObjcSelf!(void, "setSortDescriptors:", NSArray)(sortDescriptors);
	}

	NSArray sortDescriptors ()
	{
		return invokeObjcSelf!(NSArray, "sortDescriptors");
	}

	Object content ()
	{
		return invokeObjcSelf!(Object, "content");
	}

	void setContent (Object content)
	{
		return invokeObjcSelf!(void, "setContent:", Object)(content);
	}

	void add (Object sender)
	{
		return invokeObjcSelf!(void, "add:", Object)(sender);
	}

	void remove (Object sender)
	{
		return invokeObjcSelf!(void, "remove:", Object)(sender);
	}

	void addChild (Object sender)
	{
		return invokeObjcSelf!(void, "addChild:", Object)(sender);
	}

	void insert (Object sender)
	{
		return invokeObjcSelf!(void, "insert:", Object)(sender);
	}

	void insertChild (Object sender)
	{
		return invokeObjcSelf!(void, "insertChild:", Object)(sender);
	}

	bool canInsert ()
	{
		return invokeObjcSelf!(bool, "canInsert");
	}

	bool canInsertChild ()
	{
		return invokeObjcSelf!(bool, "canInsertChild");
	}

	bool canAddChild ()
	{
		return invokeObjcSelf!(bool, "canAddChild");
	}

	void insertObject (Object object, NSIndexPath indexPath)
	{
		return invokeObjcSelf!(void, "insertObject:atArrangedObjectIndexPath:", Object, NSIndexPath)(object, indexPath);
	}

	void insertObjects (NSArray objects, NSArray indexPaths)
	{
		return invokeObjcSelf!(void, "insertObjects:atArrangedObjectIndexPaths:", NSArray, NSArray)(objects, indexPaths);
	}

	void removeObjectAtArrangedObjectIndexPath (NSIndexPath indexPath)
	{
		return invokeObjcSelf!(void, "removeObjectAtArrangedObjectIndexPath:", NSIndexPath)(indexPath);
	}

	void removeObjectsAtArrangedObjectIndexPaths (NSArray indexPaths)
	{
		return invokeObjcSelf!(void, "removeObjectsAtArrangedObjectIndexPaths:", NSArray)(indexPaths);
	}

	void setAvoidsEmptySelection (bool flag)
	{
		return invokeObjcSelf!(void, "setAvoidsEmptySelection:", bool)(flag);
	}

	bool avoidsEmptySelection ()
	{
		return invokeObjcSelf!(bool, "avoidsEmptySelection");
	}

	void setPreservesSelection (bool flag)
	{
		return invokeObjcSelf!(void, "setPreservesSelection:", bool)(flag);
	}

	bool preservesSelection ()
	{
		return invokeObjcSelf!(bool, "preservesSelection");
	}

	void setSelectsInsertedObjects (bool flag)
	{
		return invokeObjcSelf!(void, "setSelectsInsertedObjects:", bool)(flag);
	}

	bool selectsInsertedObjects ()
	{
		return invokeObjcSelf!(bool, "selectsInsertedObjects");
	}

	void setAlwaysUsesMultipleValuesMarker (bool flag)
	{
		return invokeObjcSelf!(void, "setAlwaysUsesMultipleValuesMarker:", bool)(flag);
	}

	bool alwaysUsesMultipleValuesMarker ()
	{
		return invokeObjcSelf!(bool, "alwaysUsesMultipleValuesMarker");
	}

	NSArray selectedObjects ()
	{
		return invokeObjcSelf!(NSArray, "selectedObjects");
	}

	bool setSelectionIndexPaths (NSArray indexPaths)
	{
		return invokeObjcSelf!(bool, "setSelectionIndexPaths:", NSArray)(indexPaths);
	}

	NSArray selectionIndexPaths ()
	{
		return invokeObjcSelf!(NSArray, "selectionIndexPaths");
	}

	bool setSelectionIndexPath (NSIndexPath indexPath)
	{
		return invokeObjcSelf!(bool, "setSelectionIndexPath:", NSIndexPath)(indexPath);
	}

	NSIndexPath selectionIndexPath ()
	{
		return invokeObjcSelf!(NSIndexPath, "selectionIndexPath");
	}

	bool addSelectionIndexPaths (NSArray indexPaths)
	{
		return invokeObjcSelf!(bool, "addSelectionIndexPaths:", NSArray)(indexPaths);
	}

	bool removeSelectionIndexPaths (NSArray indexPaths)
	{
		return invokeObjcSelf!(bool, "removeSelectionIndexPaths:", NSArray)(indexPaths);
	}

	NSArray selectedNodes ()
	{
		return invokeObjcSelf!(NSArray, "selectedNodes");
	}

	void moveNode (NSTreeNode node, NSIndexPath indexPath)
	{
		return invokeObjcSelf!(void, "moveNode:toIndexPath:", NSTreeNode, NSIndexPath)(node, indexPath);
	}

	void moveNodes (NSArray nodes, NSIndexPath startingIndexPath)
	{
		return invokeObjcSelf!(void, "moveNodes:toIndexPath:", NSArray, NSIndexPath)(nodes, startingIndexPath);
	}

	NSString childrenKeyPathForNode (NSTreeNode node)
	{
		return invokeObjcSelf!(NSString, "childrenKeyPathForNode:", NSTreeNode)(node);
	}

	NSString countKeyPathForNode (NSTreeNode node)
	{
		return invokeObjcSelf!(NSString, "countKeyPathForNode:", NSTreeNode)(node);
	}

	NSString leafKeyPathForNode (NSTreeNode node)
	{
		return invokeObjcSelf!(NSString, "leafKeyPathForNode:", NSTreeNode)(node);
	}
}

