/**
 * Copyright: Copyright (c) 2009 Jacob Carlborg.
 * Authors: Jacob Carlborg
 * Version: Initial created: Aug 3, 2009 
 * License: $(LINK2 http://www.boost.org/LICENSE_1_0.txt, Boost Software License 1.0)
 */
module dstep.foundation.NSIndexSet;

import dstep.foundation.NSCoder;
import dstep.foundation.NSObjCRuntime;
import dstep.foundation.NSObject;
import dstep.foundation.NSRange;
import dstep.foundation.NSZone;
import dstep.objc.bridge.Bridge;
import dstep.objc.objc;

class NSMutableIndexSet : NSIndexSet
{
	mixin (ObjcWrap);
	
	this ()
	{
		typeof(this).alloc.init;
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	void addIndexes (NSIndexSet indexSet)
	{
		return invokeObjcSelf!(void, "addIndexes:", NSIndexSet)(indexSet);
	}

	void removeIndexes (NSIndexSet indexSet)
	{
		return invokeObjcSelf!(void, "removeIndexes:", NSIndexSet)(indexSet);
	}

	void removeAllIndexes ()
	{
		return invokeObjcSelf!(void, "removeAllIndexes");
	}

	void addIndex (NSUInteger value)
	{
		return invokeObjcSelf!(void, "addIndex:", NSUInteger)(value);
	}

	void removeIndex (NSUInteger value)
	{
		return invokeObjcSelf!(void, "removeIndex:", NSUInteger)(value);
	}

	void addIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(void, "addIndexesInRange:", NSRange)(range);
	}

	void removeIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(void, "removeIndexesInRange:", NSRange)(range);
	}

	void shiftIndexesStartingAtIndex (NSUInteger index, NSInteger delta)
	{
		return invokeObjcSelf!(void, "shiftIndexesStartingAtIndex:by:", NSUInteger, NSInteger)(index, delta);
	}
}

class NSIndexSet : NSObject, INSCopying, INSMutableCopying, INSCoding
{
	mixin (ObjcWrap);
	
	this ()
	{
		typeof(this).alloc.init;
	}
	
	typeof(this) init ()
	{
		return invokeObjcSelf!(typeof(this), "init");
	}

	static Object indexSet ()
	{
		return invokeObjcSuperClass!(Object, "indexSet");
	}

	static Object indexSetWithIndex (NSUInteger value)
	{
		return invokeObjcSuperClass!(Object, "indexSetWithIndex:", NSUInteger)(value);
	}

	static Object indexSetWithIndexesInRange (NSRange range)
	{
		return invokeObjcSuperClass!(Object, "indexSetWithIndexesInRange:", NSRange)(range);
	}

	Object initWithIndex (NSUInteger value)
	{
		return invokeObjcSelf!(Object, "initWithIndex:", NSUInteger)(value);
	}

	this (NSUInteger value)
	{
		typeof(this).alloc.initWithIndex(value);
	}

	Object initWithIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(Object, "initWithIndexesInRange:", NSRange)(range);
	}

	this (NSRange range)
	{
		typeof(this).alloc.initWithIndexesInRange(range);
	}

	Object initWithIndexSet (NSIndexSet indexSet)
	{
		return invokeObjcSelf!(Object, "initWithIndexSet:", NSIndexSet)(indexSet);
	}

	this (NSIndexSet indexSet)
	{
		typeof(this).alloc.initWithIndexSet(indexSet);
	}

	bool isEqualToIndexSet (NSIndexSet indexSet)
	{
		return invokeObjcSelf!(bool, "isEqualToIndexSet:", NSIndexSet)(indexSet);
	}

	NSUInteger count ()
	{
		return invokeObjcSelf!(NSUInteger, "count");
	}

	NSUInteger firstIndex ()
	{
		return invokeObjcSelf!(NSUInteger, "firstIndex");
	}

	NSUInteger lastIndex ()
	{
		return invokeObjcSelf!(NSUInteger, "lastIndex");
	}

	NSUInteger indexGreaterThanIndex (NSUInteger value)
	{
		return invokeObjcSelf!(NSUInteger, "indexGreaterThanIndex:", NSUInteger)(value);
	}

	NSUInteger indexLessThanIndex (NSUInteger value)
	{
		return invokeObjcSelf!(NSUInteger, "indexLessThanIndex:", NSUInteger)(value);
	}

	NSUInteger indexGreaterThanOrEqualToIndex (NSUInteger value)
	{
		return invokeObjcSelf!(NSUInteger, "indexGreaterThanOrEqualToIndex:", NSUInteger)(value);
	}

	NSUInteger indexLessThanOrEqualToIndex (NSUInteger value)
	{
		return invokeObjcSelf!(NSUInteger, "indexLessThanOrEqualToIndex:", NSUInteger)(value);
	}

	NSUInteger getIndexes (NSUInteger* indexBuffer, NSUInteger bufferSize, NSRangePointer range)
	{
		return invokeObjcSelf!(NSUInteger, "getIndexes:maxCount:inIndexRange:", NSUInteger*, NSUInteger, NSRangePointer)(indexBuffer, bufferSize, range);
	}

	NSUInteger countOfIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(NSUInteger, "countOfIndexesInRange:", NSRange)(range);
	}

	bool containsIndex (NSUInteger value)
	{
		return invokeObjcSelf!(bool, "containsIndex:", NSUInteger)(value);
	}

	bool containsIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(bool, "containsIndexesInRange:", NSRange)(range);
	}

	bool containsIndexes (NSIndexSet indexSet)
	{
		return invokeObjcSelf!(bool, "containsIndexes:", NSIndexSet)(indexSet);
	}

	bool intersectsIndexesInRange (NSRange range)
	{
		return invokeObjcSelf!(bool, "intersectsIndexesInRange:", NSRange)(range);
	}

	Object copyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "copyWithZone:", NSZone*)(zone);
	}

	Object mutableCopyWithZone (NSZone* zone)
	{
		return invokeObjcSelf!(Object, "mutableCopyWithZone:", NSZone*)(zone);
	}

	void encodeWithCoder (NSCoder aCoder)
	{
		return invokeObjcSelf!(void, "encodeWithCoder:", NSCoder)(aCoder);
	}

	Object initWithCoder (NSCoder aDecoder)
	{
		return invokeObjcSelf!(Object, "initWithCoder:", NSCoder)(aDecoder);
	}

	this (NSCoder aDecoder)
	{
		typeof(this).alloc.initWithCoder(aDecoder);
	}
}

